# -*- encoding: utf-8 -*-
from abjad import *


def test_spannertools_Spanner_pop_01():
    r'''Remove and return rightmost component in spanner.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    beam = Beam()
    attach(beam, voice[:])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8 ]
            }
        }
        '''
        )

    result = beam.pop()

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
            {
                g'8
                a'8
            }
        }
        '''
        )

    assert result is voice[-1]



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def map_sequence_elements_to_canonic_tuples(sequence, decrease_parts_monotonically=True):
    '''Partition `sequence` elements into canonic parts that decrease monotonically:

    ::

        >>> sequencetools.map_sequence_elements_to_canonic_tuples(
        ...     range(10))
        [(0,), (1,), (2,), (3,), (4,), (4, 1), (6,), (7,), (8,), (8, 1)]

    Partition `sequence` elements into canonic parts that increase monotonically:

    ::

        >>> sequencetools.map_sequence_elements_to_canonic_tuples(
        ...     range(10), decrease_parts_monotonically=False)
        [(0,), (1,), (2,), (3,), (4,), (1, 4), (6,), (7,), (8,), (1, 8)]

    Raise type error when `sequence` is not a list.

    Raise value error on noninteger elements in `sequence`.

    Returns list of tuples.
    '''

    if not isinstance(sequence, list):
        raise TypeError

    if not all(isinstance(x, (int, long)) for x in sequence):
        raise ValueError

    result = []

    for x in sequence:
        result.append(mathtools.partition_integer_into_canonic_parts(
            x, decrease_parts_monotonically=decrease_parts_monotonically))

    return result



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def partition_sequence_by_weights_at_most(
    sequence, 
    weights, 
    cyclic=False, 
    overhang=False,
    ):
    r'''Partition `sequence` by `weights` at most.

    ::

        >>> sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]

    ..  container:: example
    
        **Example 1.** Partition sequence once by weights at most without 
        overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights_at_most(
            ...     sequence, 
            ...     [10, 4], 
            ...     cyclic=False, 
            ...     overhang=False,
            ...     )
            [[3, 3, 3], [3]]

    ..  container:: example
    
        **Example 2.** Partition sequence once by weights at most with 
        overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights_at_most(
            ...     sequence, 
            ...     [10, 4], 
            ...     cyclic=False, 
            ...     overhang=True,
            ...     )
            [[3, 3, 3], [3], [4, 4, 4, 4, 5, 5]]

    ..  container:: example
    
        **Example 3.** Partition sequence cyclically by weights at most 
        without overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights_at_most(
            ...     sequence, 
            ...     [10, 5], 
            ...     cyclic=True, 
            ...     overhang=False,
            ...     )
            [[3, 3, 3], [3], [4, 4], [4], [4, 5], [5]]

    ..  container:: example
    
        **Example 4.** Partition sequence cyclically by weights at most 
        with overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights_at_most(
            ...     sequence, 
            ...     [10, 5], 
            ...     cyclic=True, 
            ...     overhang=True,
            ...     )
            [[3, 3, 3], [3], [4, 4], [4], [4, 5], [5]]

    Returns list of sequence objects.
    '''
    if not cyclic:
        return _partition_sequence_once_by_weights_at_most(
            sequence, 
            weights, 
            overhang=overhang,
            )
    else:
        return _partition_sequence_cyclically_by_weights_at_most(
            sequence, 
            weights, 
            overhang=overhang,
            )


def _partition_sequence_once_by_weights_at_most(
    sequence, 
    weights, 
    overhang=False,
    ):

    l_copy = sequence[:]
    result = []
    current_part = []

    for target_weight in weights:
        while True:
            try:
                x = l_copy.pop(0)
            except IndexError:
                message = 'too few elements in sequence.'
                raise PartitionError(message)
            current_weight = mathtools.weight(current_part)
            candidate_weight = current_weight + mathtools.weight([x])
            if candidate_weight < target_weight:
                current_part.append(x)
            elif candidate_weight == target_weight:
                current_part.append(x)
                result.append(current_part)
                current_part = []
                break
            elif target_weight < candidate_weight:
                if current_part:
                    result.append(current_part)
                    current_part = []
                    l_copy.insert(0, x)
                    break
                else:
                    message = 'elements in sequence too big.'
                    raise PartitionError(message)
            else:
                message = 'candidate and target weights must compare.'
                raise ValueError(message)

    if overhang:
        left_over = current_part + l_copy
        if left_over:
            result.append(left_over)

    return result


def _partition_sequence_cyclically_by_weights_at_most(
    sequence, 
    weights, 
    overhang=False,
    ):

    result = []
    current_part = []
    current_target_weight_index = 0
    current_target_weight = weights[current_target_weight_index]
    l_copy = sequence[:]

    while l_copy:
        current_target_weight = weights[current_target_weight_index % len(weights)]
        x = l_copy.pop(0)
        current_part_weight = mathtools.weight(current_part)
        candidate_part_weight = current_part_weight + mathtools.weight([x])
        if candidate_part_weight < current_target_weight:
            current_part.append(x)
        elif candidate_part_weight == current_target_weight:
            current_part.append(x)
            result.append(current_part)
            current_part = []
            current_target_weight_index += 1
        elif current_target_weight < candidate_part_weight:
            if current_part:
                l_copy.insert(0, x)
                result.append(current_part)
                current_part = []
                current_target_weight_index += 1
            else:
                message = 'elements in sequence too big.'
                raise PartitionError(message)
        else:
            message = 'candidate and target rates must compare.'
            raise ValueError(message)

    if current_part:
        if overhang:
            result.append(current_part)

    return result



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_repeat_sequence_elements_at_indices_01():
    r'''Excepted case.
    '''

    sequence_2 = list(sequencetools.repeat_sequence_elements_at_indices(range(10), [6, 7, 8], 3))
    assert sequence_2 == [0, 1, 2, 3, 4, 5, [6, 6, 6], [7, 7, 7], [8, 8, 8], 9]


def test_sequencetools_repeat_sequence_elements_at_indices_02():
    r'''Boundary cases.
    '''

    sequence_2 = list(sequencetools.repeat_sequence_elements_at_indices(range(10), [], 99))
    assert sequence_2 == [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]

    sequence_2 = list(sequencetools.repeat_sequence_elements_at_indices(range(10), range(10), 1))
    assert sequence_2 == [[0], [1], [2], [3], [4], [5], [6], [7], [8], [9]]

    sequence_2 = list(sequencetools.repeat_sequence_elements_at_indices(range(10), range(10), 2))
    assert sequence_2 == [[0, 0], [1, 1], [2, 2], [3, 3], [4, 4],
        [5, 5], [6, 6], [7, 7], [8, 8], [9, 9]]

    sequence_2 = list(sequencetools.repeat_sequence_elements_at_indices(range(10), [6, 7, 8], 0))
    assert sequence_2 == [0, 1, 2, 3, 4, 5, [], [], [], 9]



# -*- encoding: utf-8 -*-
from abjad import *


def test_lilypondfiletools_BookBlock_01():

    book_block = lilypondfiletools.BookBlock()

    assert systemtools.TestManager.compare(
        book_block,
        r'''
        \book {}
        '''
        )



# -*- encoding: utf-8 -*-
import inspect
import pytest
from abjad.tools import documentationtools


ignored_names = (
    '__dict__',
    '__init__',
    '__new__',
    '__weakref__',
    'context_name',
    'denominator',
    'duration',
    'multiplier',
    'music',
    'numerator',
    'optional_id',
    'optional_context_mod',
    'push_signature',
    'type',
    'value',
    )

ignored_classes = (
    'LilyPondLexicalDefinition',
    'LilyPondSyntacticalDefinition',
    'ReducedLyParser',
    'RhythmTreeParser',
    'SchemeParser',
    )


classes = documentationtools.list_all_abjad_classes()
@pytest.mark.parametrize('obj', classes)
def test___doc___01(obj):
    r'''All classes have a docstring. All class methods have a docstring.
    '''

    assert obj.__doc__ is not None
    if obj.__name__ in ignored_classes:
        return
    for attr in inspect.classify_class_attrs(obj):
        if attr.name in ignored_names:
            continue
        elif attr.defining_class is not obj:
            continue
        if attr.name[0].isalpha() or attr.name.startswith('__'):
            message = '{}.{}'.format(obj.__name__, attr.name)
            assert getattr(obj, attr.name).__doc__ is not None, message


functions = documentationtools.list_all_abjad_functions()
@pytest.mark.parametrize('obj', functions)
def test___doc___02(obj):
    r'''All functions have a docstring.
    '''

    assert obj.__doc__ is not None



# -*- encoding: utf-8 -*-
from abjad.tools.rhythmmakertools.DivisionBurnishedTaleaRhythmMaker import \
    DivisionBurnishedTaleaRhythmMaker


class TaleaRhythmMaker(DivisionBurnishedTaleaRhythmMaker):
    r'''Talea rhythm-maker.

    ..  container:: example

        **Example 1.** Basic usage:

        ::

            >>> maker = rhythmmakertools.TaleaRhythmMaker(
            ...     talea=[-1, 4, -2, 3],
            ...     talea_denominator=16,
            ...     prolation_addenda=[3, 4])

        ::

            >>> divisions = [(2, 8), (5, 8)]
            >>> music = maker(divisions)
            >>> music = sequencetools.flatten_sequence(music)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = Staff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, music)

        ::

            >>> show(staff) # doctest: +SKIP

    ..  container:: example

        **Example 2.** Tie split notes.

            >>> maker = rhythmmakertools.TaleaRhythmMaker(
            ...     talea=[5],
            ...     talea_denominator=16,
            ...     tie_split_notes=True)

        ::

            >>> divisions = [(2, 8), (2, 8), (2, 8), (2, 8)]
            >>> music = maker(divisions)
            >>> music = sequencetools.flatten_sequence(music)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = Staff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, music)

        ::

            >>> show(staff) # doctest: +SKIP

    Usage follows the two-step instantiate-then-call pattern shown here.
    '''

    ### INITIALIZER ###

    def __init__(
        self,
        talea=None,
        talea_denominator=None,
        prolation_addenda=None,
        secondary_divisions=None,
        talea_helper=None,
        prolation_addenda_helper=None,
        secondary_divisions_helper=None,
        beam_each_cell=False,
        beam_cells_together=False,
        tie_split_notes=False,
        ):
        lefts, middles, rights = [0], [0], [0]
        left_lengths, right_lengths = [0], [0]
        DivisionBurnishedTaleaRhythmMaker.__init__(
            self,
            talea,
            talea_denominator,
            prolation_addenda,
            lefts,
            middles,
            rights,
            left_lengths,
            right_lengths,
            secondary_divisions,
            talea_helper=talea_helper,
            prolation_addenda_helper=prolation_addenda_helper,
            secondary_divisions_helper=secondary_divisions_helper,
            beam_each_cell=beam_each_cell,
            beam_cells_together=beam_cells_together,
            tie_split_notes=tie_split_notes,
            )

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats talea rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        ::

            >>> print format(maker)
            rhythmmakertools.TaleaRhythmMaker(
                talea=[5],
                talea_denominator=16,
                prolation_addenda=[],
                secondary_divisions=[],
                beam_each_cell=False,
                beam_cells_together=False,
                tie_split_notes=True,
                )

        Returns string.
        '''
        superclass = super(TaleaRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    ### PUBLIC METHODS ###

    def __makenew__(self, *args, **kwargs):
        r'''Creates new talea rhythm-maker with `kwargs`.

        ::

            >>> new_maker = new(maker, prolation_addenda=[1])

        ::

            >>> print format(new_maker)
            rhythmmakertools.TaleaRhythmMaker(
                talea=[5],
                talea_denominator=16,
                prolation_addenda=[1],
                secondary_divisions=[],
                beam_each_cell=False,
                beam_cells_together=False,
                tie_split_notes=True,
                )

        ::

            >>> divisions = [(2, 8), (5, 8)]
            >>> music = new_maker(divisions)
            >>> music = sequencetools.flatten_sequence(music)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = Staff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, music)

        ::

            >>> show(staff) # doctest: +SKIP

        Returns new talea rhythm-maker.
        '''
        return DivisionBurnishedTaleaRhythmMaker.__makenew__(
            self, *args, **kwargs)

    def reverse(self):
        r'''Reverses talea rhythm-maker.

        ::

            >>> reversed_maker = maker.reverse()

        ::

            >>> print format(reversed_maker)
            rhythmmakertools.TaleaRhythmMaker(
                talea=[5],
                talea_denominator=16,
                prolation_addenda=[],
                secondary_divisions=[],
                beam_each_cell=False,
                beam_cells_together=False,
                tie_split_notes=True,
                )

        ::

            >>> divisions = [(2, 8), (5, 8)]
            >>> music = reversed_maker(divisions)
            >>> music = sequencetools.flatten_sequence(music)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = Staff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, music)

        ::

            >>> show(staff) # doctest: +SKIP

        Returns new talea rhythm-maker.
        '''
        return DivisionBurnishedTaleaRhythmMaker.reverse(self)



# -*- encoding: utf-8 -*-
import os
import types
from abjad.tools import abctools
from abjad.tools.documentationtools.ModuleCrawler import ModuleCrawler


class FunctionCrawler(abctools.AbjadObject):
    r'''Function crawler.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_code_root',
        '_include_private_objects',
        '_module_crawler',
        '_root_package_name',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        code_root=None,
        include_private_objects=False,
        root_package_name=None,
        ):
        self._module_crawler = ModuleCrawler(
            code_root, root_package_name=root_package_name)
        self._code_root = code_root
        self._include_private_objects = include_private_objects
        if root_package_name is None:
            self._root_package_name = self._module_crawler.root_package_name
        else:
            self._root_package_name = root_package_name

    ### SPECIAL METHODS ###

    def __call__(self):
        r'''Calls function crawler.

        Returns tuplet of functions.
        '''
        objects = []
        for module in self.module_crawler:
            name = module.__name__.split('.')[-1]
            if not self.include_private_objects and name.startswith('_'):
                continue
            if not hasattr(module, name):
                continue
            obj = getattr(module, name)
            if isinstance(obj, types.FunctionType):
                objects.append(obj)
        return tuple(sorted(objects))

    ### PUBLIC PROPERTIES ###

    @property
    def code_root(self):
        r'''Code root of function crawler.
        '''
        return self._code_root

    @property
    def include_private_objects(self):
        r'''Include private objets.
        '''
        return self._include_private_objects

    @property
    def module_crawler(self):
        r'''Module crawler.
        '''
        return self._module_crawler

    @property
    def root_package_name(self):
        r'''Root package name.
        '''
        return self._root_package_name



# -*- encoding: utf-8 -*-
from abjad.tools import *


def test_rhythmmakertools_TaleaRhythmMaker___eq___01():

    maker_1 = rhythmmakertools.TaleaRhythmMaker([-1, 2, -3, 4], 16,
        prolation_addenda=[2, 3],
        secondary_divisions=[6],
        )

    maker_2 = rhythmmakertools.TaleaRhythmMaker([-1, 2, -3, 4], 16,
        prolation_addenda=[2, 3],
        secondary_divisions=[6],
        )

    assert maker_1 == maker_1
    assert maker_1 == maker_2
    assert not maker_1 == 'foo'
    assert maker_2 == maker_1
    assert maker_2 == maker_2
    assert not maker_2 == 'foo'
    assert not 'foo' == maker_1
    assert not 'foo' == maker_2
    assert 'foo' == 'foo'



# -*- encoding: utf-8 -*-
import numbers


def truncate_runs_in_sequence(sequence):
    '''Truncate subruns of like elements in `sequence` to length ``1``:

    ::

        >>> sequencetools.truncate_runs_in_sequence([1, 1, 2, 3, 3, 3, 9, 4, 4, 4])
        [1, 2, 3, 9, 4]

    Returns empty list when `sequence` is empty:

    ::

        >>> sequencetools.truncate_runs_in_sequence([])
        []

    Raise type error when `sequence` is not a list.

    Returns new list.
    '''

    if not isinstance(sequence, list):
        raise TypeError

    assert all(isinstance(x, numbers.Number) for x in sequence)

    result = []

    if sequence:
        result.append(sequence[0])
        for element in sequence[1:]:
            if not element == result[-1]:
                result.append(element)

    return result



# -*- encoding: utf-8 -*-


def yield_all_subsequences_of_sequence(sequence, min_length=0, max_length=None):
    '''Yield all subsequences of `sequence` in lex order:

    ::

        >>> list(sequencetools.yield_all_subsequences_of_sequence([0, 1, 2]))
        [[], [0], [0, 1], [0, 1, 2], [1], [1, 2], [2]]

    Yield all subsequences of `sequence` greater than or equal to `min_length`
    in lex order:

    ::

        >>> list(sequencetools.yield_all_subsequences_of_sequence(
        ...     [0, 1, 2, 3, 4], min_length=3))
        [[0, 1, 2], [0, 1, 2, 3], [0, 1, 2, 3, 4], [1, 2, 3], [1, 2, 3, 4], [2, 3, 4]]

    Yield all subsequences of `sequence` less than or equal to `max_length`
    in lex order:

    ::

        >>> for subsequence in sequencetools.yield_all_subsequences_of_sequence(
        ...     [0, 1, 2, 3, 4], max_length=3):
        ...     subsequence
        []
        [0]
        [0, 1]
        [0, 1, 2]
        [1]
        [1, 2]
        [1, 2, 3]
        [2]
        [2, 3]
        [2, 3, 4]
        [3]
        [3, 4]
        [4]

    Yield all subsequences of `sequence` greater than or equal to `min_length` and
    less than or equal to `max_length` in lex order:

    ::

        >>> list(sequencetools.yield_all_subsequences_of_sequence(
        ...     [0, 1, 2, 3, 4], min_length=3, max_length=3))
        [[0, 1, 2], [1, 2, 3], [2, 3, 4]]

    Returns generator of newly created `sequence` slices.
    '''

    len_sequence = len(sequence)

    if max_length is None:
        max_length = len_sequence

    for i in range(len_sequence):
        start_j = min_length + i
        stop_j = min(max_length + i, len_sequence) + 1
        for j in range(start_j, stop_j):
            if i < j or i == 0:
                subsequence = sequence[i:j]
                yield subsequence



# -*- encoding: utf-8 -*-
#! /usr/bin/env python

if __name__ == '__main__':

    from abjad import *
    from abjad.demos import mozart

    lilypond_file = mozart.make_mozart_lilypond_file()
    show(lilypond_file)



# -*- encoding: utf-8 -*-
from abjad.tools.spannertools.Spanner import Spanner


# TODO: remove and just use Spanner instead with overrides
class TextScriptSpanner(Spanner):
    r'''A text script spanner.

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> spanner = spannertools.TextScriptSpanner()
        >>> attach(spanner, staff[:])
        >>> override(spanner).text_script.color = 'red'
        >>> markup = markuptools.Markup(r'\italic { espressivo }', Up)
        >>> attach(markup, staff[1])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            \override TextScript #'color = #red
            c'8
            d'8 ^ \markup { \italic { espressivo } }
            e'8
            f'8
            \revert TextScript #'color
        }

    Override LilyPond TextScript grob.
    '''

    ### INITIALIZER ###

    def __init__(
        self, 
        components=None,
        overrides=None,
        ):
        Spanner.__init__(
            self, 
            components,
            overrides=overrides,
            )

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        pass



import types


def attach(indicator, component_expression, scope=None):
    r'''Attaches `indicator` to `component_expression`.

    Derives scope from the default scope of `indicator` 
    when `scope` is none.

    Returns none.
    '''
    from abjad.tools import indicatortools
    from abjad.tools import scoretools
    from abjad.tools import spannertools

    if hasattr(indicator, '_attach'):
        prototype = (spannertools.Spanner, scoretools.GraceContainer)
        assert isinstance(indicator, prototype), repr(indicator)
        assert scope is None
        indicator._attach(component_expression)
        return

    component = component_expression
    #assert isinstance(component, scoretools.Component), repr(component)
    prototype = (scoretools.Component, spannertools.Spanner)
    assert isinstance(component, prototype), repr(component)

    if isinstance(indicator, indicatortools.IndicatorExpression):
        scope = scope or indicator.scope
        indicator._detach()
        indicator = indicator.indicator

    if hasattr(indicator, '_default_scope'):
        scope = scope or indicator._default_scope

    expression = indicatortools.IndicatorExpression(
        indicator, 
        component,
        scope,
        )
    expression._bind_to_component(component)



# -*- encoding: utf-8 -*-
from abjad.tools.lilypondfiletools.NonattributedBlock \
    import NonattributedBlock


class ScoreBlock(NonattributedBlock):
    r'''Abjad model of LilyPond input file score block:

    ::

        >>> score_block = lilypondfiletools.ScoreBlock()

    ::

        >>> score_block
        ScoreBlock()

    ::

        >>> score_block.append(Staff([]))
        >>> print format(score_block)
        \score {
            \new Staff {
            }
        }

    ScoreBlocks does not format when empty, as this generates a
    parser error in LilyPond:

    ::

        >>> score_block = lilypondfiletools.ScoreBlock()
        >>> format(score_block) == ''
        True

    Returns score block.
    '''

    ### INITIALIZER ###

    def __init__(self):
        NonattributedBlock.__init__(self)
        self._escaped_name = r'\score'
        self._is_formatted_when_empty = False

    ### PRIVATE PROPERTIES ###

    @property
    def _format_pieces(self):
        from abjad.tools import markuptools
        from abjad.tools import scoretools
        result = []
        if not len(self):
            if self._is_formatted_when_empty:
                result.append(r'%s {}' % self._escaped_name)
        else:
            result.append(r'%s {' % self._escaped_name)
            if len(self) == 1 and \
                isinstance(self[0], (scoretools.Leaf, markuptools.Markup)):
                result.append('\t{')
                result.extend(
                    ['\t\t' + piece for piece in self[0]._format_pieces])
                result.append('\t}')
            else:
                for x in self:
                    if isinstance(x, str):
                        result.append('\t%s' % x)
                    elif hasattr(x, '_get_format_pieces'):
                        result.extend(
                            ['\t' + piece for piece in x._get_format_pieces()])
                    else:
                        result.extend(
                            ['\t' + piece for piece in x._format_pieces])
            result.append('}')
        return result



# -*- encoding: utf-8 -*-
from abjad.tools.sequencetools.flatten_sequence import flatten_sequence
from abjad.tools.sequencetools.zip_sequences_without_truncation \
	import zip_sequences_without_truncation


def interlace_sequences(*sequences):
    '''Interlace `sequences`:

    ::

        >>> k = range(100, 103)
        >>> l = range(200, 201)
        >>> m = range(300, 303)
        >>> n = range(400, 408)
        >>> sequencetools.interlace_sequences(k, l, m, n)
        [100, 200, 300, 400, 101, 301, 401, 102, 302, 402, 403, 404, 405, 406, 407]

    Returns list.
    '''

    zipped_sequences = zip_sequences_without_truncation(*sequences)
    flattened_sequences = flatten_sequence(zipped_sequences, depth=1)

    return flattened_sequences



# -*- encoding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_Chord_note_heads_01():
    r'''Chord returns note heads as an immutable tuple.
    '''

    chord = Chord("<d' e' f'>4")
    note_heads = chord.note_heads

    assert isinstance(note_heads, scoretools.NoteHeadInventory)
    assert len(note_heads) == 3


def test_scoretools_Chord_note_heads_02():
    r'''Note heads of chords with equivalent pitches compare equal.
    '''

    chord_1 = Chord("<d' e' f'>4")
    chord_2 = Chord("<d' e' f'>4")

    assert chord_1.note_heads == chord_2.note_heads


def test_scoretools_Chord_note_heads_03():
    r'''Set note heads with pitch numbers.
    '''

    chord = Chord('<>4')
    chord.note_heads = [4, 3, 2]

    assert format(chord) == "<d' ef' e'>4"


def test_scoretools_Chord_note_heads_04():
    r'''Set note heads with pitches.
    '''

    chord = Chord('<>4')
    chord.note_heads = [
        NamedPitch(4),
        NamedPitch(3),
        NamedPitch(2),
        ]

    assert format(chord) == "<d' ef' e'>4"


def test_scoretools_Chord_note_heads_05():
    r'''Set note heads with both pitches and pitch numbers.
    '''

    chord = Chord('<>4')
    chord.note_heads = [
        NamedPitch(4),
        3,
        NamedPitch(2),
        ]

    assert format(chord) == "<d' ef' e'>4"


def test_scoretools_Chord_note_heads_06():
    r'''Set note heads with a LilyPond input string.
    '''

    chord = Chord("<c'>4")
    chord.note_heads = "c' d' e'"

    assert format(chord) == "<c' d' e'>4"


def test_scoretools_Chord_note_heads_07():
    r'''Set note head color with the LilyPond tweak reservoir.
    '''

    chord = Chord("<ef' cs'' f''>4")
    chord.note_heads[0].tweak.color = 'red'
    chord.note_heads[1].tweak.color = 'green'
    chord.note_heads[2].tweak.color = 'blue'

    assert systemtools.TestManager.compare(
        chord,
        r'''
        <
            \tweak #'color #red
            ef'
            \tweak #'color #green
            cs''
            \tweak #'color #blue
            f''
        >4
        '''
        )



# -*- encoding: utf-8 -*-
import collections
import numbers
import re
from abjad.tools import datastructuretools
from abjad.tools import indicatortools
from abjad.tools.abctools import AbjadObject
from abjad.tools.pitchtools.Pitch import Pitch


# TODO: make iterable so that for x in PitchRange works
class PitchRange(AbjadObject):
    r"""A pitch range.

    ::

        >>> pitch_range = pitchtools.PitchRange(-12, 36,
        ...     pitch_range_name='four-octave range')
        >>> print format(pitch_range)
        pitchtools.PitchRange(
            '[C3, C7]',
            pitch_range_name='four-octave range',
            pitch_range_name_markup=markuptools.Markup(
                ('four-octave range',)
                ),
            )

    ::

        >>> show(pitch_range) # doctest: +SKIP

    Initalize from pitch numbers, pitch names, pitch instances,
    one-line reprs or other pitch range objects.

    Pitch ranges implement equality testing against other pitch ranges.

    Pitch ranges test less than, greater than, less-equal and
    greater-equal against pitches.

    Pitch ranges do not sort relative to other pitch ranges.

    Pitch ranges are immutable.
    """

    ### CLASS VARIABLES ###

    _symbolic_pitch_range_string_regex_body = '''
        ([\[(])         # open bracket or open parenthesis
        ({}|{}|-?\d+)   # pitch
        ,               # comma
        [ ]*            # any amount of whitespace
        ({}|{}|-?\d+)   # pitch
        ([\])])         # close bracket or close parenthesis
        '''.format(
            Pitch._pitch_class_octave_number_regex_body,
            Pitch._pitch_name_regex_body,
            Pitch._pitch_class_octave_number_regex_body,
            Pitch._pitch_name_regex_body,
            )

    _symbolic_pitch_range_string_regex = re.compile(
        '^{}$'.format(_symbolic_pitch_range_string_regex_body),
        re.VERBOSE,
        )

    __slots__ = (
        '_start',
        '_stop',
        '_pitch_range_name',
        '_pitch_range_name_markup',
        )

    ### INITIALIZER ###

    def __init__(self, *args, **kwargs):
        from abjad.tools import pitchtools
        if len(args) == 1 and isinstance(args[0], type(self)):
            if args[0].start_pitch_is_included_in_range:
                boundedness = 'inclusive'
            else:
                boundedness = 'exclusive'
            start = (args[0].start_pitch, boundedness)
            self._start = start
            assert isinstance(args[0], type(self)), repr(args[0])
            if args[0].stop_pitch_is_included_in_range:
                boundedness = 'inclusive'
            else:
                boundedness = 'exclusive'
            stop = (args[0].stop_pitch, boundedness)
            self._stop = stop
        elif len(args) == 1 and isinstance(args[0], str):
            self._initialize_by_symbolic_pitch_range_string(*args)
        elif len(args) == 1 and isinstance(args[0], collections.Sequence):
            start, stop = args[0]
            type(self).__init__(self, start, stop)
        elif len(args) == 0:
            start_pitch = pitchtools.NamedPitch('A0')
            stop_pitch = pitchtools.NamedPitch('C8')
            self._start = (start_pitch, 'inclusive')
            self._stop = (stop_pitch, 'inclusive')
        else:
            assert len(args) == 2, repr(args)
            start, stop = args
            if start is None:
                start = start
            elif isinstance(start, (int, long, float, str)):
                pitch = pitchtools.NamedPitch(start)
                start = (pitch, 'inclusive')
            else:
                assert len(start) == 2, repr(start)
                pitch, containment = start
                assert containment in ('inclusive', 'exclusive')
                pitch = pitchtools.NamedPitch(pitch)
                start = (pitch, containment)
            self._start = start
            if stop is None:
                stop = stop
            elif isinstance(stop, (int, long, float, str)):
                pitch = pitchtools.NamedPitch(stop)
                stop = (pitch, 'inclusive')
            else:
                assert len(stop) == 2, repr(stop)
                pitch, containment = stop
                assert containment in ('inclusive', 'exclusive'), \
                    repr(containment)
                pitch = pitchtools.NamedPitch(pitch)
                stop = (pitch, containment)
            self._stop = stop
        pitch_range_name = kwargs.get('pitch_range_name')
        self._pitch_range_name = pitch_range_name
        pitch_range_name_markup = kwargs.get('pitch_range_name_markup')
        self._pitch_range_name_markup = pitch_range_name_markup

    ### SPECIAL METHODS ###

    def __contains__(self, arg):
        r'''True when pitch range contains `arg`. Otherwise false.

        Returns boolean.
        '''
        from abjad.tools import pitchtools
        from abjad.tools import scoretools
        from abjad.tools.agenttools.InspectionAgent import inspect
        if hasattr(arg, '_has_effective_indicator') and \
            arg._has_effective_indicator(indicatortools.IsUnpitched):
            return True
        elif isinstance(arg, (int, long, float)):
            pitch = pitchtools.NamedPitch(arg)
            return self._contains_pitch(pitch)
        elif isinstance(arg, pitchtools.NamedPitch):
            return self._contains_pitch(arg)
        elif isinstance(arg, scoretools.Note):
            sounding_pitch = inspect(arg).get_sounding_pitch()
            return self._contains_pitch(sounding_pitch)
        elif isinstance(arg, scoretools.Chord):
            sounding_pitches = inspect(arg).get_sounding_pitches()
            return all(self._contains_pitch(x) for x in sounding_pitches)
        elif isinstance(arg, (scoretools.Rest, scoretools.Skip)):
            return True
        elif isinstance(arg, scoretools.Container):
            return all(x in self for x in arg.select_leaves())
        else:
            pitches = pitchtools.list_named_pitches_in_expr(arg)
            if pitches:
                return all(self._contains_pitch(x) for x in pitches)
            else:
                try:
                    return all(self._contains_pitch(x) for x in arg)
                except TypeError:
                    return False
        return False

    def __eq__(self, arg):
        r'''True when `arg` is a pitch range with start and stop equal to those
        of this pitch range. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, type(self)):
            if self._start == arg._start:
                if self._stop == arg._stop:
                    return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats pitch range.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __ge__(self, arg):
        r'''True when start pitch of pitch range is greater than or equal to
        `arg`. Otherwise false.

        Returns boolean.
        '''
        from abjad.tools import pitchtools
        try:
            pitch = pitchtools.NamedPitch(arg)
            if self.start_pitch is None:
                return False
            return pitch <= self.start_pitch
        except (TypeError, ValueError):
            return False

    def __gt__(self, arg):
        r'''True when start pitch of pitch range is greater than `arg`.
        Otherwise false.

        Returns boolean.
        '''
        from abjad.tools import pitchtools
        try:
            pitch = pitchtools.NamedPitch(arg)
            if self.start_pitch is None:
                return False
            return pitch < self.start_pitch
        except (TypeError, ValueError):
            return False

    def __illustrate__(self):
        r'''Illustrates pitch range.

        ::

            >>> show(pitch_range) # doctest: +SKIP

        Returns LilyPond file.
        '''
        from abjad.tools import durationtools
        from abjad.tools import lilypondfiletools
        from abjad.tools import indicatortools
        from abjad.tools import markuptools
        from abjad.tools import pitchtools
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import attach
        from abjad.tools.topleveltools import iterate
        from abjad.tools.topleveltools import override
        start_pitch_clef = pitchtools.suggest_clef_for_named_pitches(
            self.start_pitch)
        stop_pitch_clef = pitchtools.suggest_clef_for_named_pitches(
            self.stop_pitch)
        start_note = scoretools.Note(self.start_pitch, 1)
        stop_note = scoretools.Note(self.stop_pitch, 1)
        glissando = spannertools.Glissando()
        if start_pitch_clef == stop_pitch_clef:
            if start_pitch_clef == indicatortools.Clef('bass'):
                bass_staff = scoretools.Staff()
                attach(indicatortools.Clef('bass'), bass_staff)
                bass_staff.extend([start_note, stop_note])
                attach(glissando, bass_staff.select_leaves())
                score = scoretools.Score([bass_staff])
            else:
                treble_staff = scoretools.Staff()
                attach(indicatortools.Clef('treble'), treble_staff)
                treble_staff.extend([start_note, stop_note])
                attach(glissando, treble_staff.select_leaves())
                score = scoretools.Score([treble_staff])
        else:
            result = scoretools.make_empty_piano_score()
            score, treble_staff, bass_staff = result
            bass_staff.extend([start_note, stop_note])
            treble_staff.extend(scoretools.Skip(1) * 2)
            attach(glissando, bass_staff.select_leaves())
            attach(indicatortools.StaffChange(treble_staff), bass_staff[1])
        for leaf in iterate(score).by_class(scoretools.Leaf):
            attach(durationtools.Multiplier(1, 4), leaf)
        override(score).bar_line.stencil = False
        override(score).span_bar.stencil = False
        override(score).glissando.thickness = 2
        override(score).time_signature.stencil = False
        lilypond_file = lilypondfiletools.make_basic_lilypond_file(score)
        lilypond_file.header_block.tagline = markuptools.Markup('""')
        return lilypond_file

    def __le__(self, arg):
        r'''True when stop pitch of pitch-range is less than or equal to `arg`.
        Otherwise false.

        Returns boolean.
        '''
        from abjad.tools import pitchtools
        try:
            pitch = pitchtools.NamedPitch(arg)
            if self.stop_pitch is None:
                return False
            return self.stop_pitch <= pitch
        except (TypeError, ValueError):
            return False

    def __lt__(self, arg):
        r'''True when stop pitch of pitch-range is less than `arg`. Otherwise
        false.

        Returns boolean.
        '''
        from abjad.tools import pitchtools
        try:
            pitch = pitchtools.NamedPitch(arg)
            if self.stop_pitch is None:
                return False
            return self.stop_pitch < pitch
        except (TypeError, ValueError):
            return False

    def __ne__(self, arg):
        r'''True when pitch range does not equal `arg`. Otherwise false.

        Returns boolean.
        '''
        return not self == arg

    ### PRIVATE CLASS VARIABLES ###

    _start_punctuation_to_inclusivity_string = {
        '[': 'inclusive',
        '(': 'exclusive',
        }

    _stop_punctuation_to_inclusivity_string = {
        ']': 'inclusive',
        ')': 'exclusive',
        }

    ### PRIVATE PROPERTIES ###

    @property
    def _close_bracket_string(self):
        if self.stop_pitch_is_included_in_range:
            return ']'
        else:
            return ')'

    @property
    def _one_line_menuing_summary(self):
        return self.one_line_named_pitch_repr

    @property
    def _open_bracket_string(self):
        if self.start_pitch_is_included_in_range:
            return '['
        else:
            return '('

    @property
    def _repr_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            keyword_argument_names=(
                'pitch_range_name',
                'pitch_range_name_markup',
                ),
            positional_argument_values=(
                self.one_line_named_pitch_repr,
                ),
            )

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            keyword_argument_names=(
                'pitch_range_name',
                'pitch_range_name_markup',
                ),
            positional_argument_values=(
                self.one_line_named_pitch_repr,
                ),
            )

    ### PRIVATE METHODS ###

    def _contains_pitch(self, pitch):
        from abjad.tools import pitchtools
        if isinstance(pitch, numbers.Number):
            pitch = pitchtools.NamedPitch(pitch)
        elif isinstance(pitch, str):
            pitch = pitchtools.NamedPitch(pitch)
        if self._start is None and self._stop is None:
            return True
        elif self._start is None:
            if self.stop_pitch_is_included_in_range:
                return pitch <= self.stop_pitch
            else:
                return pitch < self.stop_pitch
        elif self._stop is None:
            if self.start_pitch_is_included_in_range:
                return self.start_pitch <= pitch
            else:
                return self.start_pitch < pitch
        else:
            if self.start_pitch_is_included_in_range:
                if self.stop_pitch_is_included_in_range:
                    return self.start_pitch <= pitch <= self.stop_pitch
                else:
                    return self.start_pitch <= pitch < self.stop_pitch
            else:
                if self.stop_pitch_is_included_in_range:
                    return self.start_pitch < pitch <= self.stop_pitch
                else:
                    return self.start_pitch < pitch < self.stop_pitch

    def _initialize_by_symbolic_pitch_range_string(self, symbolic_pitch_range_string):
        from abjad.tools import pitchtools
        match = self._symbolic_pitch_range_string_regex.match(
            symbolic_pitch_range_string)
        if match is None:
            message = 'can not instantiate pitch range: {!r}'
            message = message.format(symbolic_pitch_range_string)
            raise ValueError(message)
        groups = match.groups()
        start_punctuation = groups[0]
        start_pitch_string = groups[1]
        stop_pitch_string = groups[8]
        stop_punctuation = groups[-1]
        start_inclusivity_string = \
            self._start_punctuation_to_inclusivity_string[start_punctuation]
        stop_inclusivity_string = \
            self._stop_punctuation_to_inclusivity_string[stop_punctuation]
        start_pair = (start_pitch_string, start_inclusivity_string)
        stop_pair = (stop_pitch_string, stop_inclusivity_string)
        type(self).__init__(self, start_pair, stop_pair)

    ### PUBLIC METHODS ###

    @classmethod
    def is_symbolic_pitch_range_string(cls, expr):
        '''True when `expr` is a symbolic pitch range string. Otherwise false:

        ::

            >>> pitchtools.PitchRange.is_symbolic_pitch_range_string(
            ...     '[A0, C8]')
            True

        The regex that underlies this predicate matches against two
        comma-separated pitches enclosed in some combination of square
        brackets and round parentheses.

        Returns boolean.
        '''
        if not isinstance(expr, str):
            return False
        return bool(cls._symbolic_pitch_range_string_regex.match(expr))

    ### PUBLIC PROPERTIES ###

    @property
    def one_line_named_pitch_repr(self):
        r'''One-line named pitch representation of pitch range.

        ::

            >>> pitch_range.one_line_named_pitch_repr
            '[C3, C7]'

        Returns string.
        '''
        result = []
        result.append(self._open_bracket_string)
        if self.start_pitch:
            result.append(self.start_pitch.pitch_class_octave_label)
            result.append(', ')
        if self.stop_pitch:
            result.append(self.stop_pitch.pitch_class_octave_label)
        result.append(self._close_bracket_string)
        result = ''.join(result)
        return result

    @property
    def one_line_numbered_pitch_repr(self):
        r'''One-line numbered pitch representation of pitch range.

        ::

            >>> pitch_range.one_line_numbered_pitch_repr
            '[-12, 36]'

        Returns string.
        '''
        result = []
        result.append(self._open_bracket_string)
        result.append(str(self.start_pitch.pitch_number))
        result.append(', ')
        result.append(str(self.stop_pitch.pitch_number))
        result.append(self._close_bracket_string)
        result = ''.join(result)
        return result

    # TODO: change to custom_identifier
    @property
    def pitch_range_name(self):
        r'''Name of pitch range.

        ::

            >>> pitch_range.pitch_range_name
            'four-octave range'

        Returns string or none.
        '''
        return self._pitch_range_name
            
    # TODO: change to custom_identifier_markup
    @property
    def pitch_range_name_markup(self):
        r'''Pitch range name markup.

        ::

            >>> pitch_range.pitch_range_name_markup
            Markup(('four-octave range',))

        Default to `pitch_range_name` when `pitch_range_name_markup`
        not set explicitly.

        Returns markup or none.
        '''
        from abjad.tools import markuptools
        if self._pitch_range_name_markup:
            return self._pitch_range_name_markup
        elif self.pitch_range_name:
            return markuptools.Markup(self.pitch_range_name)

    @property
    def start_pitch(self):
        r'''Start pitch of pitch range.

        ::

            >>> pitch_range.start_pitch
            NamedPitch('c')

        Returns pitch.
        '''
        if self._start is None:
            return None
        return self._start[0]

    @property
    def start_pitch_is_included_in_range(self):
        r'''True when start pitch is included in range.
        Otherwise false:

        ::

            >>> pitch_range.start_pitch_is_included_in_range
            True

        Returns boolean.
        '''
        if self._start is None:
            return True
        return self._start[1] == 'inclusive'

    @property
    def stop_pitch(self):
        r"""Stop pitch of pitch range.

        ::

            >>> pitch_range.stop_pitch
            NamedPitch("c''''")

        Returns pitch.
        """
        if self._stop is None:
            return None
        return self._stop[0]

    @property
    def stop_pitch_is_included_in_range(self):
        r'''True when stop pitch is included in range.
        Otherwise false:

        ::

            >>> pitch_range.stop_pitch_is_included_in_range
            True

        Returns boolean.
        '''
        if self._stop is None:
            return True
        return self._stop[1] == 'inclusive'



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_zip_sequences_without_truncation_01():
    r'''Zip and do not truncate to the length of the shortest list.
    '''

    sequence_2 = sequencetools.zip_sequences_without_truncation([1, 2, 3, 4], [11, 12, 13])
    assert sequence_2 == [(1, 11), (2, 12), (3, 13), (4,)]


def test_sequencetools_zip_sequences_without_truncation_02():
    r'''Zip and do not truncate to the length of the shortest list.
    '''

    sequence_2 = sequencetools.zip_sequences_without_truncation([1, 2, 3], [11, 12, 13, 14])
    assert sequence_2 == [(1, 11), (2, 12), (3, 13), (14,)]



# -*- encoding: utf-8 -*-
from abjad import *


def test_datastructuretools_StatalServer___eq___01():

    statal_server_1 = datastructuretools.StatalServer([0, 1, 2, 3, 4])
    statal_server_2 = datastructuretools.StatalServer([0, 1, 2, 3, 4])
    statal_server_3 = datastructuretools.StatalServer([0, 1, 2, 3])

    assert     statal_server_1 == statal_server_1
    assert     statal_server_1 == statal_server_2
    assert not statal_server_1 == statal_server_3
    assert     statal_server_2 == statal_server_1
    assert     statal_server_2 == statal_server_2
    assert not statal_server_2 == statal_server_3
    assert not statal_server_3 == statal_server_1
    assert not statal_server_3 == statal_server_2
    assert     statal_server_3 == statal_server_3



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_splice_new_elements_between_sequence_elements_01():
    r'''Insert a copy of the elements of sequence 2 between
    each of the elements of sequence 1.
    '''

    sequence_1 = [0, 1, 2, 3, 4]
    sequence_2 = ['A', 'B']

    t = sequencetools.splice_new_elements_between_sequence_elements(
        sequence_1, sequence_2)
    assert t == [0, 'A', 'B', 1, 'A', 'B', 2, 'A', 'B', 3, 'A', 'B', 4]

    t = sequencetools.splice_new_elements_between_sequence_elements(
        sequence_1, sequence_2, overhang=(0, 1))
    assert t == [0, 'A', 'B', 1, 'A', 'B', 2, 'A', 'B', 3, 'A', 'B', 4, 'A', 'B']

    t = sequencetools.splice_new_elements_between_sequence_elements(
        sequence_1, sequence_2, overhang=(1, 0))
    assert t == ['A', 'B', 0, 'A', 'B', 1, 'A', 'B', 2, 'A', 'B', 3, 'A', 'B', 4]

    t = sequencetools.splice_new_elements_between_sequence_elements(
        sequence_1, sequence_2, overhang=(1, 1))
    assert t == ['A', 'B', 0, 'A', 'B', 1, 'A', 'B', 2, 'A', 'B', 3, 'A', 'B', 4, 'A', 'B']



# -*- encoding: utf-8 -*-
from abjad.tools.quantizationtools.QEvent import QEvent


class TerminalQEvent(QEvent):
    r'''The terminal event in a series of ``QEvents``:

        >>> q_event = quantizationtools.TerminalQEvent(1000)
        >>> print format(q_event)
        quantizationtools.TerminalQEvent(
            offset=durationtools.Offset(1000, 1),
            )

    Carries no significance outside the context of a ``QEventSequence``.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_offset',
        )

    ### INITIALIZER ###

    def __init__(self, offset=0):
        QEvent.__init__(self, offset=offset)

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''True when `expr` is a terminal q-event with offset equal to that of
        this terminal q-event. Otherwise false.

        Returns boolean.
        '''
        if type(self) == type(expr) and \
            self.offset == expr.offset:
            return True
        return False



# -*- encoding: utf-8 -*-
import os


class TestManager(object):
    r'''Manages test logic.
    '''

    @staticmethod
    def apply_additional_layout(lilypond_file):
        r'''Configures multiple-voice rhythmic staves in `lilypond_file`.

        Operates in place.
        
        Returns none.
        '''
        from abjad.tools import layouttools
        from abjad.tools import scoretools
        from abjad.tools import topleveltools
        # configure multiple-voice rhythmic staves
        for staff in topleveltools.iterate(lilypond_file.score_block[0]
            ).by_class(scoretools.Staff):
            if staff.is_simultaneous:
                assert len(staff) == 2
                voice_1 = staff[0]
                topleveltools.override(voice_1).note_head.Y_offset = 0.5
                topleveltools.override(voice_1).stem.direction = Up
                voice_2 = staff[1]
                topleveltools.override(voice_2).note_head.Y_offset = -0.5
                topleveltools.override(voice_2).stem.direction = Down
                spacing_vector = layouttools.make_spacing_vector(0, 0, 6, 0)
                topleveltools.override(staff
                    ).vertical_axis_group.staff_staff_spacing = spacing_vector
        # provide more space between staves with pitched notes
        for staff in topleveltools.iterate(lilypond_file.score_block[0]
            ).by_class(scoretools.Staff):
            if not isinstance(staff, scoretools.RhythmicStaff):
                for context_block in lilypond_file.layout_block.context_blocks:
                    if context_block.context_name == 'StaffGroup':
                        #staff_group_context_block = context_block
                        break
                else:
                    message = 'no staff group context block found.'
                    raise Exception(message)
                spacing_vector = layouttools.make_spacing_vector(0, 0, 6, 0)
                topleveltools.override(context_block
                    ).vertical_axis_group.staff_staff_spacing = spacing_vector
            break

    @staticmethod
    def compare(string_1, string_2):
        r'''Compares `string_1` to `string_2`.

        Massage newlines.

        Returns boolean.
        '''
        if not isinstance(string_1, str):
            string_1 = format(string_1)
        split_lines = string_2.split('\n')
        if not split_lines[0] or split_lines[0].isspace():
            split_lines.pop(0)
        if not split_lines[-1] or split_lines[-1].isspace():
            split_lines.pop(-1)
        for indent_width, character in enumerate(split_lines[0]):
            if character != ' ':
                break
        tab_string = 4 * ' '
        massaged_lines = []
        for split_line in split_lines:
            massaged_line = split_line[indent_width:]
            #massaged_line = massaged_line.replace(tab_string, '\t')
            massaged_lines.append(massaged_line)
        massaged_string = '\n'.join(massaged_lines)
        return string_1.replace('\t', '    ') == massaged_string

    @staticmethod
    def get_current_function_name():
        r'''Gets current function name.

        ::

            >>> def foo():
            ...        function_name = systemtools.TestManager.get_current_function_name()
            ...        print 'Function name is {!r}.'.format(function_name)

        ::

            >>> foo()
            Function name is 'foo'.

        Call this function within the implementation of any ofther function.

        Returns enclosing function name as a string or else none.
        '''
        import inspect
        stack = inspect.stack()
        # per the inspect module doc page ...
        try:
            parent_frame_record = stack[1]
            parent_frame = parent_frame_record[0]
            parent_frame_info = inspect.getframeinfo(parent_frame)
            parent_frame_function_name = parent_frame_info.function
            return parent_frame_function_name
        # ... destroy frame to avoid reference cycle
        finally:
            del stack

    @staticmethod
    def read_test_output(full_file_name, current_function_name):
        r'''Reads test output.

        Returns list.
        '''
        segment_ly_file_name = '{}.ly'.format(current_function_name)
        directory_name = os.path.dirname(full_file_name)
        segment_ly_path_name = os.path.join(
            directory_name,
            segment_ly_file_name,
            )
        return file(segment_ly_path_name, 'r').read()

    @staticmethod
    def test_function_name_to_title_lines(test_function_name):
        r'''Changes `test_function_name` to title lines.

        Returns list.
        '''
        from abjad.tools import sequencetools
        title_lines = []
        test_function_name = test_function_name[5:]
        if '__' in test_function_name:
            left_half, right_half = test_function_name.split('__')
            left_half = left_half.replace('_', ' ')
            title_lines.append(left_half)
            parts = right_half.split('_')
        else:
            parts = test_function_name.split('_')
        test_number = int(parts[-1])
        parts.pop(-1)
        if parts[0][0].isupper() and 1 < len(parts[0]):
            title_lines.append(parts.pop(0))
        lengths = [len(part) for part in parts]
        if 35 < sum(lengths):
            halves = sequencetools.partition_sequence_by_ratio_of_weights(
                lengths,
                [1, 1],
                )
            left_count = len(halves[0])
            right_count = len(halves[-1])
            assert left_count + right_count == len(lengths)
            left_parts = parts[:left_count]
            title_lines.append(' '.join(left_parts))
            right_parts = parts[-right_count:]
            right_parts.append(str(test_number))
            title_lines.append(' '.join(right_parts))
        else:
            title_words = ' '.join(parts)
            if 'schematic example' in title_words:
                space = ''
            else:
                space = ' '
            title = '{}{}{}'.format(title_words, space, test_number)
            title_lines.append(title)
        return title_lines

    @staticmethod
    def write_test_output(
        output,
        full_file_name,
        test_function_name,
        cache_ly=False,
        cache_pdf=False,
        go=False,
        render_pdf=False,
        ):
        r'''Writes test output.

        Returns none.
        '''
        from abjad.tools import lilypondfiletools
        from abjad.tools import markuptools
        from abjad.tools import schemetools
        from abjad.tools import scoretools
        from abjad.tools import systemtools
        from abjad.tools import topleveltools
        if go:
            cache_ly = cache_pdf = render_pdf = True
        if not any([cache_ly, cache_pdf, render_pdf]):
            return
        if isinstance(output, scoretools.Score):
            lilypond_file = \
                lilypondfiletools.make_floating_time_signature_lilypond_file(
                    output)
            TestManager.apply_additional_layout(lilypond_file)
            score = output
        elif isinstance(output, lilypondfiletools.LilyPondFile):
            lilypond_file = output
            score = lilypond_file.score_block[0]
        else:
            raise TypeError(output)
        title_lines = TestManager.test_function_name_to_title_lines(
            test_function_name)
        lilypond_file.header_block.title = \
            markuptools.make_centered_title_markup(
                title_lines,
                font_size=6,
                vspace_before=2,
                vspace_after=4,
                )
        moment = schemetools.SchemeMoment((1, 48))
        topleveltools.contextualize(lilypond_file.score
            ).proportionalNotationDuration = moment
        parent_directory_name = os.path.dirname(full_file_name)
        if render_pdf:
            topleveltools.show(lilypond_file)
        if cache_pdf:
            file_name = '{}.pdf'.format(test_function_name)
            pdf_path_name = os.path.join(parent_directory_name, file_name)
            topleveltools.persist(lilypond_file).as_pdf(pdf_path_name)
        if cache_ly:
            file_name = '{}.ly'.format(test_function_name)
            ly_path_name = os.path.join(parent_directory_name, file_name)
            file(ly_path_name, 'w').write(format(score))



# -*- encoding: utf-8 -*-


def snake_case_to_lower_camel_case(string):
    '''Change underscore-delimited lowercase `string` to lowercamelcase:

    ::

        >>> string = 'bass_figure_alignment_positioning'
        >>> stringtools.snake_case_to_lower_camel_case(string)
        'bassFigureAlignmentPositioning'

    Returns string.
    '''

    parts = string.split('_')
    first_part = parts[:1]
    rest_parts = parts[1:]
    rest_parts = [part.title() for part in rest_parts]
    all_parts = first_part + rest_parts
    return ''.join(all_parts)



# -*- encoding: utf-8 -*-
import os
from abjad.tools.developerscripttools.DeveloperScript import DeveloperScript


class MakeNewFunctionTemplateScript(DeveloperScript):
    r'''Makes new function stub files.

    ..  shell::

        ajv new function --help

    '''

    ### PUBLIC PROPERTIES ###

    @property
    def alias(self):
        r'''Alias of script.

        Returns ``'function'``.
        '''
        return 'function'

    @property
    def long_description(self):
        r'''Long description of script.

        Returns string or none.
        '''
        return None

    @property
    def scripting_group(self):
        r'''Scripting group of script.

        Returns ``'new'``.
        '''
        return 'new'

    @property
    def short_description(self):
        r'''Short description of script.

        Returns string.
        '''
        return 'Make a new function template file.'

    @property
    def version(self):
        r'''Version of script.

        Returns float.
        '''
        return 1.0

    ### PRIVATE METHODS ###

    def _get_function_names_in_tools_package(self, root, tools_package_name):
        path = os.path.join(root, tools_package_name)
        return tuple(sorted([x[:-3] for x in os.listdir(path)
            if x.endswith('.py') and not x.startswith('__')]))

    def _get_function_text(self, function_name):
        return [
            'def {}():'.format(function_name),
            '    pass'
        ]

    def _get_tools_package_names(self, root):
        names = []
        for x in os.listdir(root):
            if os.path.isdir(os.path.join(root, x)):
                if not x.startswith(('_', '.')):
                    names.append(x)
        return tuple(sorted(names))

    ### PUBLIC METHODS ###

    def process_args(self, args):
        r'''Processes `args`.

        Returns none.
        '''
        if args.name.count('.') != 1:
            message = 'Error: {!r} not in tools_package.function format.'
            message = message.format(args.name)
            print message
            return
        root = args.path
        tools_package_name, function_name = args.name.split('.')
        if tools_package_name not in self._get_tools_package_names(root):
            message = 'Error: {!r} is not a valid tools package.'
            message = message.format(tools_package_name)
            print message
            return
        if function_name in self._get_function_names_in_tools_package(
            root, tools_package_name):
            message = 'Error: {!r} already exists in {!r}'
            message = message.format(function_name, tools_package_name)
            print message
            return
        package_path = os.path.join(root, tools_package_name)
        module_path = os.path.join(package_path, '{}.py'.format(function_name))
        module_text = '\n'.join(self._get_function_text(function_name)) + '\n'
        with open(module_path, 'w') as f:
            f.write(module_text)

    def setup_argument_parser(self, parser):
        r'''Sets up argument `parser`.
        
        Returns none.
        '''
        from abjad import abjad_configuration
        parser.add_argument('name',
            help='tools package qualified function name'
            )
        path_group = parser.add_mutually_exclusive_group(required=True)
        path_group.add_argument('-X', '--experimental',
            action='store_const',
            const=os.path.join(
                abjad_configuration.abjad_experimental_directory_path, 
                'tools'),
            dest='path',
            help='use the Abjad experimental tools path',
            )
        path_group.add_argument('-M', '--mainline',
            action='store_const',
            const=os.path.join(
                abjad_configuration.abjad_directory_path, 'tools'),
            dest='path',
            help='use the Abjad mainline tools path',
            )



# -*- encoding: utf-8 -*-
import abc
import types


class AbjadObject(object):
    '''Abstract base class from which all custom classes should inherit.

    Abjad objects compare equal only with equal object IDs.
    '''

    ### CLASS VARIABLES ###

    __metaclass__ = abc.ABCMeta

    __slots__ = ()

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''Is true when ID of `expr` equals ID of Abjad object.
        Otherwise false.

        Returns boolean.
        '''
        return id(self) == id(expr)

    def __format__(self, format_specification=''):
        r'''Formats object.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __getstate__(self):
        r'''Gets object state.
        '''
        if hasattr(self, '__dict__'):
            return vars(self)
        state = {}
        for class_ in type(self).__mro__:
            for slot in getattr(class_, '__slots__', ()):
                state[slot] = getattr(self, slot, None)
        return state

    def __ne__(self, expr):
        r'''Is true when Abjad object does not equal `expr`.
        Otherwise false.

        Returns boolean.
        '''
        return not self == expr

    def __repr__(self):
        r'''Gets interpreter representation of Abjad object.

        Returns string.
        '''
        from abjad.tools import systemtools
        return systemtools.StorageFormatManager.get_repr_format(self)

    def __setstate__(self, state):
        r'''Sets object state.
        '''
        for key, value in state.iteritems():
            setattr(self, key, value)

    ### PRIVATE PROPERTIES ###

    @property
    def _one_line_menuing_summary(self):
        return str(self)

    @property
    def _repr_specification(self):
        return self._storage_format_specification.__makenew__(
            is_indented=False,
            )

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(self)

    ### PRIVATE METHODS ###

    def _debug(self, value, annotation=None, blank=False):
        if annotation is None:
            print 'debug: {!r}'.format(value)
        else:
            print 'debug ({}): {!r}'.format(annotation, value)
        if blank:
            print ''

    def _debug_values(self, values, annotation=None, blank=True):
        if values:
            for value in values:
                self._debug(value, annotation=annotation)
            if blank:
                print ''
        else:
            self._debug(repr(values), annotation=annotation)
            if blank:
                print ''



# -*- encoding: utf-8 -*-
from abjad.tools import stringtools
from abjad.tools.spannertools.Spanner import Spanner


class Beam(Spanner):
    r'''A beam.

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8 g'2")
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            c'8
            d'8
            e'8
            f'8
            g'2
        }

    ::

        >>> beam = spannertools.Beam()
        >>> attach(beam, staff[:4])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            c'8 [
            d'8
            e'8
            f'8 ]
            g'2
        }

    '''

    ### INITIALIZER ###

    def __init__(self, components=None, direction=None, overrides=None):
        Spanner.__init__(self,
        components,
        overrides=overrides,
        )
        self.direction = direction

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        #Spanner._copy_keyword_args(self, new)
        new.direction = self.direction

    def _format_right_of_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            if self.direction is not None:
                result.append('%s [' % self.direction)
            else:
                result.append('[')
        if self._is_my_last_leaf(leaf):
            result.append(']')
        return result

    ### PUBLIC METHODS ###

    @staticmethod
    def is_beamable_component(expr):
        '''True when `expr` is a beamable component. Otherwise false.

        ::

            >>> staff = Staff(r"r32 a'32 ( [ gs'32 fs''32 \staccato f''8 ) ]")
            >>> staff.extend(r"r8 e''8 ( ef'2 )")
            >>> show(staff) # doctest: +SKIP

        ::

            >>> for leaf in staff.select_leaves():
            ...     beam = spannertools.Beam
            ...     result = beam.is_beamable_component(leaf)
            ...     print '{:<8}\t{}'.format(leaf, result)
            ...
            r32     False
            a'32    True
            gs'32   True
            fs''32  True
            f''8    True
            r8      False
            e''8    True
            ef'2    False

        Returns boolean.
        '''
        from abjad.tools import scoretools
        from abjad.tools import scoretools
        if isinstance(expr, (scoretools.Note, scoretools.Chord)):
            if 0 < expr.written_duration.flag_count:
                return True
        return False

    ### PUBLIC PROPERTIES ###

    @property
    def direction(self):
        r'''Gets and sets direction of beam.

        Returns up or down.
        '''
        return self._direction

    @direction.setter
    def direction(self, arg):
        self._direction = \
            stringtools.arg_to_tridirectional_lilypond_symbol(arg)



# -*- encoding: utf-8 -*-
lilypond_version = "2.17.9"

engravers = set([
    "Accidental_engraver",
    "Ambitus_engraver",
    "Arpeggio_engraver",
    "Auto_beam_engraver",
    "Axis_group_engraver",
    "Balloon_engraver",
    "Bar_engraver",
    "Bar_number_engraver",
    "Beam_collision_engraver",
    "Beam_engraver",
    "Beam_performer",
    "Bend_engraver",
    "Break_align_engraver",
    "Breathing_sign_engraver",
    "Chord_name_engraver",
    "Chord_tremolo_engraver",
    "Clef_engraver",
    "Cluster_spanner_engraver",
    "Collision_engraver",
    "Completion_heads_engraver",
    "Completion_rest_engraver",
    "Concurrent_hairpin_engraver",
    "Control_track_performer",
    "Cue_clef_engraver",
    "Custos_engraver",
    "Default_bar_line_engraver",
    "Dot_column_engraver",
    "Dots_engraver",
    "Double_percent_repeat_engraver",
    "Drum_note_performer",
    "Drum_notes_engraver",
    "Dynamic_align_engraver",
    "Dynamic_engraver",
    "Dynamic_performer",
    "Engraver",
    "Episema_engraver",
    "Extender_engraver",
    "Figured_bass_engraver",
    "Figured_bass_position_engraver",
    "Fingering_column_engraver",
    "Fingering_engraver",
    "Font_size_engraver",
    "Footnote_engraver",
    "Forbid_line_break_engraver",
    "Fretboard_engraver",
    "Glissando_engraver",
    "Grace_beam_engraver",
    "Grace_engraver",
    "Grace_spacing_engraver",
    "Grid_line_span_engraver",
    "Grid_point_engraver",
    "Grob_pq_engraver",
    "Hara_kiri_engraver",
    "Horizontal_bracket_engraver",
    "Hyphen_engraver",
    "Instrument_name_engraver",
    "Instrument_switch_engraver",
    "Keep_alive_together_engraver",
    "Key_engraver",
    "Key_performer",
    "Laissez_vibrer_engraver",
    "Ledger_line_engraver",
    "Ligature_bracket_engraver",
    "Lyric_engraver",
    "Lyric_performer",
    "Mark_engraver",
    "Measure_grouping_engraver",
    "Melody_engraver",
    "Mensural_ligature_engraver",
    "Metronome_mark_engraver",
    "Multi_measure_rest_engraver",
    "New_dynamic_engraver",
    "New_fingering_engraver",
    "Note_head_line_engraver",
    "Note_heads_engraver",
    "Note_name_engraver",
    "Note_performer",
    "Note_spacing_engraver",
    "Ottava_spanner_engraver",
    "Output_property_engraver",
    "Page_turn_engraver",
    "Paper_column_engraver",
    "Parenthesis_engraver",
    "Part_combine_engraver",
    "Percent_repeat_engraver",
    "Phrasing_slur_engraver",
    "Piano_pedal_align_engraver",
    "Piano_pedal_engraver",
    "Piano_pedal_performer",
    "Pitch_squash_engraver",
    "Pitched_trill_engraver",
    "Pure_from_neighbor_engraver",
    "Repeat_acknowledge_engraver",
    "Repeat_tie_engraver",
    "Rest_collision_engraver",
    "Rest_engraver",
    "Rhythmic_column_engraver",
    "Scheme_engraver",
    "Script_column_engraver",
    "Script_engraver",
    "Script_row_engraver",
    "Separating_line_group_engraver",
    "Slash_repeat_engraver",
    "Slur_engraver",
    "Slur_performer",
    "Spacing_engraver",
    "Span_arpeggio_engraver",
    "Span_bar_engraver",
    "Span_bar_stub_engraver",
    "Spanner_break_forbid_engraver",
    "Staff_collecting_engraver",
    "Staff_performer",
    "Staff_symbol_engraver",
    "Stanza_number_align_engraver",
    "Stanza_number_engraver",
    "Stem_engraver",
    "System_start_delimiter_engraver",
    "Tab_note_heads_engraver",
    "Tab_staff_symbol_engraver",
    "Tab_tie_follow_engraver",
    "Tempo_performer",
    "Text_engraver",
    "Text_spanner_engraver",
    "Tie_engraver",
    "Tie_performer",
    "Time_signature_engraver",
    "Time_signature_performer",
    "Timing_translator",
    "Translator",
    "Trill_spanner_engraver",
    "Tuplet_engraver",
    "Tweak_engraver",
    "Vaticana_ligature_engraver",
    "Vertical_align_engraver",
    "Volta_engraver",
])



# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject
from abjad.tools.lilypondfiletools.DateTimeToken import DateTimeToken
from abjad.tools.lilypondfiletools.LilyPondLanguageToken \
    import LilyPondLanguageToken
from abjad.tools.lilypondfiletools.LilyPondVersionToken \
    import LilyPondVersionToken


class LilyPondFile(AbjadObject, list):
    r'''A LilyPond input file.

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> lilypond_file = lilypondfiletools.make_basic_lilypond_file(staff)
        >>> lilypond_file.file_initial_user_comments.append(
        ...     'File construct as an example.')
        >>> lilypond_file.file_initial_user_comments.append(
        ...     'Parts shown here for positioning.')
        >>> lilypond_file.file_initial_user_includes.append(
        ...     'external-settings-file-1.ly')
        >>> lilypond_file.file_initial_user_includes.append(
        ...     'external-settings-file-2.ly')
        >>> lilypond_file.default_paper_size = 'letter', 'portrait'
        >>> lilypond_file.global_staff_size = 16
        >>> lilypond_file.header_block.composer = \
        ...     markuptools.Markup('Josquin')
        >>> lilypond_file.header_block.title = \
        ...     markuptools.Markup('Missa sexti tonus')
        >>> lilypond_file.layout_block.indent = 0
        >>> lilypond_file.layout_block.left_margin = 15
        >>> lilypond_file.paper_block.oddFooterMarkup = \
        ...     markuptools.Markup('The odd-page footer')
        >>> lilypond_file.paper_block.evenFooterMarkup = \
        ...     markuptools.Markup('The even-page footer')

    ..  doctest::

        >>> print format(lilypond_file)
        % ...

        % File construct as an example.
        % Parts shown here for positioning.

        \version "2..."
        \include "english.ly"

        \include "external-settings-file-1.ly"
        \include "external-settings-file-2.ly"

        #(set-default-paper-size "letter" 'portrait)
        #(set-global-staff-size 16)

        \header {
            composer = \markup { Josquin }
            title = \markup { Missa sexti tonus }
        }

        \layout {
            indent = #0
            left-margin = #15
        }

        \paper {
            evenFooterMarkup = \markup { The even-page footer }
            oddFooterMarkup = \markup { The odd-page footer }
        }

        \new Staff {
            c'8
            d'8
            e'8
            f'8
        }

    '''

    def __init__(self):
        list.__init__(self)
        self._file_initial_system_comments = []
        self._file_initial_system_comments.append(DateTimeToken())
        self._file_initial_user_comments = []
        self._file_initial_system_includes = []
        self._file_initial_system_includes.append(LilyPondVersionToken())
        self._file_initial_system_includes.append(LilyPondLanguageToken())
        self._file_initial_user_includes = []
        self.default_paper_size = None
        self.global_staff_size = None

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats LilyPond file.

        Returns string.
        '''
        if format_specification in ('', 'lilypond'):
            return self._lilypond_format
        return str(self)

    def __illustrate__(self):
        r'''Illustrates LilyPond file.

        Returns LilyPond file unchanged.
        '''
        return self

    def __repr__(self):
        r'''Gets interpreter representation of LilyPond file.

        Returns string.
        '''
        if hasattr(self, 'score_block') and 1 <= len(self.score_block):
            return '%s(%s)' % (type(self).__name__, self.score_block[0])
        else:
            return '%s()' % type(self).__name__

    ### PRIVATE PROPERTIES ###

    @property
    def _format_pieces(self):
        result = []
        result.extend(self._formatted_file_initial_system_comments)
        result.extend(self._formatted_file_initial_user_comments)
        result.extend(self._formatted_file_initial_system_includes)
        result.extend(self._formatted_file_initial_user_includes)
        result.extend(self._formatted_file_initial_scheme_settings)
        result.extend(self._formatted_blocks)
        return result

    @property
    def _formatted_blocks(self):
        result = []
        for x in self:
            if '_lilypond_format' in dir(x) and not isinstance(x, str):
                lilypond_format = format(x)
                if lilypond_format:
                    result.append(lilypond_format)
            else:
                result.append(str(x))
        return result

    @property
    def _formatted_file_initial_scheme_settings(self):
        result = []
        default_paper_size = self.default_paper_size
        if default_paper_size is not None:
            dimension, orientation = default_paper_size
            result.append("#(set-default-paper-size \"%s\" '%s)" % (
                dimension, orientation))
        global_staff_size = self.global_staff_size
        if global_staff_size is not None:
            result.append('#(set-global-staff-size %s)' % global_staff_size)
        if result:
            result = ['\n'.join(result)]
        return result

    @property
    def _formatted_file_initial_system_comments(self):
        result = []
        for x in self.file_initial_system_comments:
            if '_lilypond_format' in dir(x) and not isinstance(x, str):
                lilypond_format = format(x)
                if lilypond_format:
                    result.append('%% %s' % format(x))
            else:
                result.append('%% %s' % str(x))
        if result:
            result = ['\n'.join(result)]
        return result

    @property
    def _formatted_file_initial_system_includes(self):
        result = []
        for file_initial_include in self.file_initial_system_includes:
            if isinstance(file_initial_include, str):
                result.append(r'\include "%s"' % file_initial_include)
            else:
                result.append(format(file_initial_include))
        if result:
            result = ['\n'.join(result)]
        return result

    @property
    def _formatted_file_initial_user_comments(self):
        result = []
        for x in self.file_initial_user_comments:
            if '_lilypond_format' in dir(x) and not isinstance(x, str):
                lilypond_format = format(x)
                if lilypond_format:
                    result.append('%% %s' % format(x))
            else:
                result.append('%% %s' % str(x))
        if result:
            result = ['\n'.join(result)]
        return result

    @property
    def _formatted_file_initial_user_includes(self):
        result = []
        for file_initial_include in self.file_initial_user_includes:
            if isinstance(file_initial_include, str):
                result.append(r'\include "%s"' % file_initial_include)
            else:
                result.append(format(file_initial_include))
        if result:
            result = ['\n'.join(result)]
        return result

    @property
    def _lilypond_format(self):
        return '\n\n'.join(self._format_pieces)
        
    ### PUBLIC PROPERTIES ###

    @property
    def default_paper_size(self):
        r'''LilyPond default paper size.
        '''
        return self._default_paper_size


    @default_paper_size.setter
    def default_paper_size(self, args):
        # #(set-default-paper-size "11x17" 'landscape)
        assert args is None or len(args) == 2
        self._default_paper_size = args


    @property
    def file_initial_system_comments(self):
        r'''List of file-initial system comments.
        '''
        return self._file_initial_system_comments


    @file_initial_system_comments.setter
    def file_initial_system_comments(self, arg):
        if isinstance(arg, list):
            self._file_initial_system_comments = arg
        else:
            raise TypeError


    @property
    def file_initial_system_includes(self):
        r'''List of file-initial system include commands.
        '''
        return self._file_initial_system_includes


    @file_initial_system_includes.setter
    def file_initial_system_includes(self, arg):
        if isinstance(arg, list):
            self._file_initial_system_includes = arg
        else:
            raise TypeError


    @property
    def file_initial_user_comments(self):
        r'''List of file-initial user comments.
        '''
        return self._file_initial_user_comments


    @file_initial_user_comments.setter
    def file_initial_user_comments(self, arg):
        if isinstance(arg, list):
            self._file_initial_user_comments = arg
        else:
            raise TypeError


    @property
    def file_initial_user_includes(self):
        r'''List of file-initial user include commands.
        '''
        return self._file_initial_user_includes


    @file_initial_user_includes.setter
    def file_initial_user_includes(self, arg):
        if isinstance(arg, list):
            self._file_initial_user_includes = arg
        else:
            raise TypeError


    @property
    def global_staff_size(self):
        r'''LilyPond global staff size.
        '''
        return self._global_staff_size


    @global_staff_size.setter
    def global_staff_size(self, arg):
        assert isinstance(arg, (int, float, long, type(None)))
        self._global_staff_size = arg



# -*- encoding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools.datastructuretools.TypedList import TypedList


class PitchArrayInventory(TypedList):
    r'''Ordered collection of pitch arrays:

        >>> array_1 = pitcharraytools.PitchArray([
        ...   [1, (2, 1), ([-2, -1.5], 2)],
        ...   [(7, 2), (6, 1), 1]])

    ::

        >>> array_2 = pitcharraytools.PitchArray([
        ...   [1, 1, 1],
        ...   [1, 1, 1]])

    ::

        >>> arrays = [array_1, array_2]
        >>> inventory = pitcharraytools.PitchArrayInventory(arrays)

    ::

        >>> print format(inventory)
        pitcharraytools.PitchArrayInventory(
            [
                pitcharraytools.PitchArray(),
                pitcharraytools.PitchArray(),
                ]
            )

    '''

    def to_score(self):
        r'''Make score from pitch arrays in inventory:

        ::

            >>> array_1 = pitcharraytools.PitchArray([
            ...   [1, (2, 1), ([-2, -1.5], 2)],
            ...   [(7, 2), (6, 1), 1]])

        ::

            >>> array_2 = pitcharraytools.PitchArray([
            ...   [1, 1, 1],
            ...   [1, 1, 1]])

        ::

            >>> arrays = [array_1, array_2]
            >>> inventory = pitcharraytools.PitchArrayInventory(arrays)

        ::

            >>> score = inventory.to_score()

        ..  doctest::

            >>> print format(score)
            \new Score <<
                \new StaffGroup <<
                    \new Staff {
                        {
                            \time 4/8
                            r8
                            d'8
                            <bf bqf>4
                        }
                        {
                            \time 3/8
                            r8
                            r8
                            r8
                        }
                    }
                    \new Staff {
                        {
                            \time 4/8
                            g'4
                            fs'8
                            r8
                        }
                        {
                            \time 3/8
                            r8
                            r8
                            r8
                        }
                    }
                >>
            >>

        ::

            >>> show(score) # doctest: +SKIP

        Create one staff per pitch-array row.

        Returns score.
        '''
        score = scoretools.Score([])
        staff_group = scoretools.StaffGroup([])
        score.append(staff_group)
        number_staves = self[0].depth
        staves = scoretools.Staff([]) * number_staves
        staff_group.extend(staves)
        for pitch_array in self:
            measures = pitch_array.to_measures()
            for staff, measure in zip(staves, measures):
                staff.append(measure)
        return score



# -*- encoding: utf-8 -*-
from abjad.tools import abctools
from abjad.tools import schemetools
from abjad.tools import stringtools
from abjad.tools.abctools.AbjadObject import AbjadObject


class Markup(AbjadObject):
    r'''Abjad model of LilyPond markup.

    Initializes from string:

    ::

        >>> markup = markuptools.Markup(r'\bold { "This is markup text." }')

    ::

        >>> markup
        Markup((MarkupCommand('bold', ['This is markup text.']),))

    ..  doctest::

        >>> print format(markup)
        \markup { \bold { "This is markup text." } }

    ::

        >>> show(markup) # doctest: +SKIP

    Initializes any markup from existing markup:

    ::

        >>> markup_1 = markuptools.Markup('foo', direction=Up)
        >>> markup_2 = markuptools.Markup(markup_1, direction=Down)

    ..  doctest::

        >>> print format(markup_1)
        ^ \markup { foo }

    ..  doctest::

        >>> print format(markup_2) # doctest: +SKIP
        _ \markup { foo }

    Attach markup to score components by calling them on the component:

    ::

        >>> note = Note("c'4")

    ::

        >>> markup = markuptools.Markup(
        ...     r'\italic { "This is also markup text." }', direction=Up)

    ::

        >>> attach(markup, note)

    ..  doctest::

        >>> print format(note)
        c'4 ^ \markup { \italic { "This is also markup text." } }

    ::

        >>> show(note) # doctest: +SKIP

    Set `direction` to ``Up``, ``Down``, ``'neutral'``,
    ``'^'``, ``'_'``, ``'-'`` or None.

    Markup objects are immutable.

    Returns markup instance.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_contents',
        '_direction',
        '_format_slot',
        '_markup_name',
        )

    _default_positional_input_arguments = (
        repr(r'\bold { "This is markup text." }'),
        )

    ### INITIALIZER ###

    def __init__(self, argument, direction=None, markup_name=None):
        from abjad.tools import lilypondparsertools
        from abjad.tools import markuptools
        if not argument:
            contents = None
        # WARNING: running the LilyPondParser here on a regular basis
        #          by setting (for example) argument='' in the intializer
        #          slows performance of the entire system by up to 25%!
        elif isinstance(argument, str):
            to_parse = r'\markup {{ {} }}'.format(argument)
            parsed = lilypondparsertools.LilyPondParser()(to_parse)
            if all(isinstance(x, str) for x in parsed.contents):
                contents = (' '.join(parsed.contents),)
            else:
                contents = tuple(parsed.contents)
        elif isinstance(argument, markuptools.MarkupCommand):
            contents = (argument,)
        elif isinstance(argument, type(self)):
            contents = tuple(argument._contents)
            direction = direction or argument._direction
            markup_name = markup_name or argument._markup_name
        elif isinstance(argument, (list, tuple)) and 0 < len(argument):
            contents = []
            for arg in argument:
                if isinstance(arg, (str, markuptools.MarkupCommand)):
                    contents.append(arg)
                else:
                    contents.append(str(arg))
            contents = tuple(contents)
        else:
            contents = (str(argument),)
        self._contents = contents
        self._format_slot = 'right'
        self._markup_name = markup_name
        direction = \
            stringtools.arg_to_tridirectional_ordinal_constant(direction)
        self._direction = direction

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        r'''Copies markup.

        Returns new markup.
        '''
        return type(self)(
            self._contents,
            direction=self._direction,
            markup_name=self._markup_name,
            )

    def __eq__(self, expr):
        r'''True when `expr` is a markup with format equal to that
        of this markup. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            if format(self) == format(expr):
                return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats markup.

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.
        Interprets `''` equal to `'lilypond'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'lilypond'):
            return self._lilypond_format
        elif format_specification == 'storage':
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __hash__(self):
        r'''Hashes markup.

        Returns integer.
        '''
        return hash((type(self).__name__, self.contents))

    def __illustrate__(self):
        r'''Illustrates markup.

        Returns LilyPond file.
        '''
        from abjad.tools import lilypondfiletools
        from abjad.tools import markuptools
        lilypond_file = lilypondfiletools.make_basic_lilypond_file()
        lilypond_file.header_block.tagline = markuptools.Markup('""')
        lilypond_file.append(self)
        return lilypond_file

    def __str__(self):
        r'''String representation of markup.

        Returns string.
        '''
        return self._lilypond_format

    ### PRIVATE PROPERTIES ###

    @property
    def _format_pieces(self):
        return self._get_format_pieces()

    @property
    def _lilypond_format(self):
        return '\n'.join(self._get_format_pieces())

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            positional_argument_values=(
                self.contents,
                ),
            )

    ### PUBLIC PROPERTIES ###

    @property
    def contents(self):
        r'''Tuple of contents of markup.

        ::

            >>> markup = \
            ...     markuptools.Markup(r'\bold { "This is markup text." }')
            >>> markup.contents
            (MarkupCommand('bold', ['This is markup text.']),)

        Returns string
        '''
        return self._contents

    @property
    def direction(self):
        r'''Direction of markup.

        Returns ordinal constant.
        '''
        return self._direction

    @property
    def markup_name(self):
        r'''Name of markup.

        ::

            >>> markup = markuptools.Markup(
            ...     r'\bold { allegro ma non troppo }',
            ...     markup_name='non troppo')

        ::

            >>> markup.markup_name
            'non troppo'

        Returns string or none.
        '''
        return self._markup_name

    ### PRIVATE METHODS ###

    def _get_format_pieces(self):
        indent = '\t'
        direction = ''
        if self.direction is not None:
            direction = stringtools.arg_to_tridirectional_lilypond_symbol(
                self.direction)
        # None
        if self.contents is None:
            return [r'\markup { }']
        # a single string
        if len(self.contents) == 1 and isinstance(self.contents[0], str):
            content = self.contents[0]
            if '"' in content:
                content = schemetools.Scheme.format_scheme_value(content)
            if direction:
                return [r'{} \markup {{ {} }}'.format(direction, content)]
            return [r'\markup {{ {} }}'.format(content)]
        # multiple strings or markup commands
        if direction:
            pieces = [r'{} \markup {{'.format(direction)]
        else:
            pieces = [r'\markup {']
        for content in self.contents:
            if isinstance(content, str):
                content = schemetools.Scheme.format_scheme_value(content)
                pieces.append('{}{}'.format(indent, content))
            else:
                pieces.extend(['{}{}'.format(indent, x) for x in
                    content._get_format_pieces()])
        pieces.append('{}}}'.format(indent))
        return pieces



# -*- encoding: utf-8 -*-
import inspect
from abjad.tools import durationtools
from abjad.tools.abctools import AbjadObject


class QEventProxy(AbjadObject):
    r'''Proxies a `QEvent`, mapping that QEvent's offset with the range of 
    its beatspan to the range 0-1.

    ::

        >>> q_event = quantizationtools.PitchedQEvent(130, [0, 1, 4])
        >>> proxy = quantizationtools.QEventProxy(q_event, 0.5)
        >>> print format(proxy, 'storage')
        quantizationtools.QEventProxy(
            q_event=quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(130, 1),
                pitches=(
                    pitchtools.NamedPitch("c'"),
                    pitchtools.NamedPitch("cs'"),
                    pitchtools.NamedPitch("e'"),
                    ),
                ),
            offset=durationtools.Offset(1, 2),
            )

    Not composer-safe.

    Used internally by ``Quantizer``.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_offset', 
        '_q_event',
        )

    ### INITIALIZER ###

    def __init__(self, *args):
        from abjad.tools import quantizationtools
        if len(args) == 2:
            q_event, offset = args[0], durationtools.Offset(args[1])
            assert isinstance(q_event, quantizationtools.QEvent)
            assert 0 <= offset <= 1
        elif len(args) == 3:
            q_event, minimum, maximum = args[0], \
                durationtools.Offset(args[1]), \
                durationtools.Offset(args[2])
            assert isinstance(q_event, quantizationtools.QEvent)
            assert minimum <= q_event.offset <= maximum
            offset = (q_event.offset - minimum) / (maximum - minimum)
        elif len(args) == 0:
            q_event = None
            offset = durationtools.Offset(0)
        else:
            message = 'can not initialize {}: {!r}.'
            message = message.format(type(self).__name__, args)
            raise ValueError(message)
        self._q_event = q_event
        self._offset = offset

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''True when `expr` is a q-event proxy with offset and q-event equal
        to those of this q-event proxy. Otherwise false.

        Returns boolean.
        '''
        if type(self) == type(expr):
            if self.offset == expr.offset:
                if self.q_event == expr.q_event:
                    return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats q-event.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __getnewargs__(self):
        r'''Gets new arguments.

        Returns tuple.
        '''
        return (self.q_event, self.offset)

    def __getstate__(self):
        r'''Gets state.
        '''
        state = {}
        for current_class in inspect.getmro(type(self)):
            if hasattr(current_class, '__slots__'):
                for slot in current_class.__slots__:
                    if slot not in state:
                        state[slot] = getattr(self, slot)
        return state

    def __setstate__(self, state):
        r'''Sets state.
        '''
        for key, value in state.iteritems():
            setattr(self, key, value)

    ### PRIVATE PROPERTIES ###

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        keyword_argument_names = (
            'q_event',
            'offset',
            )
        return systemtools.StorageFormatSpecification(
            self,
            keyword_argument_names=keyword_argument_names,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def index(self):
        r'''Index of q-event proxy.
        '''
        return self._q_event.index

    @property
    def offset(self):
        r'''Offset of q-event proxy.
        '''
        return self._offset

    @property
    def q_event(self):
        r'''Q-event of q-event proxy.
        '''
        return self._q_event



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def test_mathtools_Ratio___eq___01():

    ratio_1 = mathtools.Ratio(1, 2, 1)
    ratio_2 = mathtools.Ratio(1, 2, 1)
    ratio_3 = mathtools.Ratio(2, 3, 3)

    assert ratio_1 == ratio_1
    assert ratio_1 == ratio_2
    assert not ratio_1 == ratio_3
    assert ratio_2 == ratio_1
    assert ratio_2 == ratio_2
    assert not ratio_2 == ratio_3
    assert not ratio_3 == ratio_1
    assert not ratio_3 == ratio_2
    assert ratio_3 == ratio_3


def test_mathtools_Ratio___eq___02():
    r'''Comparison works with tuples.
    '''

    assert mathtools.Ratio(1, 2, 1) == (1, 2, 1)



# -*- encoding: utf-8 -*-
from abjad.tools.mathtools.get_shared_numeric_sign \
	import get_shared_numeric_sign


def join_subsequences_by_sign_of_subsequence_elements(sequence):
    '''Join subsequences in `sequence` by sign:

    ::

        >>> sequence = [[1, 2], [3, 4], [-5, -6, -7], [-8, -9, -10], [11, 12]]
        >>> sequencetools.join_subsequences_by_sign_of_subsequence_elements(sequence)
        [[1, 2, 3, 4], [-5, -6, -7, -8, -9, -10], [11, 12]]

    ::

        >>> sequence = [[1, 2], [], [], [3, 4, 5], [6, 7]]
        >>> sequencetools.join_subsequences_by_sign_of_subsequence_elements(sequence)
        [[1, 2], [], [3, 4, 5, 6, 7]]

    Returns new list.
    '''

    if not isinstance(sequence, list):
        raise TypeError

    if not all(isinstance(x, list) for x in sequence):
        raise TypeError

    if any(get_shared_numeric_sign(x) is None for x in sequence):
        raise ValueError

    result = []

    for sublist in sequence:
        try:
            previous_sublist = result[-1]
            if get_shared_numeric_sign(previous_sublist) == get_shared_numeric_sign(sublist):
                previous_sublist.extend(sublist)
            else:
                result.append(sublist[:])
        except IndexError:
            result.append(sublist[:])

    return result



# -*- encoding: utf-8 -*-
import copy
from abjad.tools import durationtools
from abjad.tools.scoretools.Leaf import Leaf


class Skip(Leaf):
    r'''A LilyPond skip.

    ..  container:: example

        ::

            >>> skip = scoretools.Skip((3, 16))
            >>> skip
            Skip('s8.')

        ..  doctest::

            >>> print format(skip)
            s8.

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    _default_positional_input_arguments = (
        repr('s4'),
        )

    ### INITIALIZER ###

    def __init__(self, arg):
        from abjad.tools import lilypondparsertools
        if isinstance(arg, str):
            input = '{{ {} }}'.format(arg)
            parsed = lilypondparsertools.LilyPondParser()(input)
            assert len(parsed) == 1 and isinstance(parsed[0], Leaf)
            arg = parsed[0]
        if isinstance(arg, Leaf):
            written_duration = arg.written_duration
        elif not isinstance(arg, str):
            written_duration = arg
        else:
            message = 'can not initialize skip from {!r}.'
            message = message.format(arg)
            raise ValueError(message)
        Leaf.__init__(self, written_duration)
        if isinstance(arg, Leaf):
            self._copy_override_and_set_from_leaf(arg)

    ### PRIVATE PROPERTIES ###

    @property
    def _body(self):
        result = []
        result.append('s%s' % self._formatted_duration)
        return result

    @property
    def _compact_representation(self):
        return 's%s' % self._formatted_duration



# -*- encoding: utf-8 -*-
from abjad.tools.abjadbooktools.OutputFormat import OutputFormat


class HTMLOutputFormat(OutputFormat):
    r'''HTML output format.
    '''

    ### INITIALIZER ###

    def __init__(self):
        code_block_opening = '<pre class="abjad">\n'
        code_block_closing = '</pre>\n'
        code_indent = 0
        image_block = '<img alt="" src="images/{}.png"/>\n'
        image_format = 'png'
        OutputFormat.__init__(
            self, 
            code_block_opening, 
            code_block_closing,
            code_indent, 
            image_block, 
            image_format,
            )



# -*- encoding: utf-8 -*-
from abjad.tools import abctools
from abjad.tools import durationtools
from abjad.tools import sequencetools
from abjad.tools.topleveltools import iterate


class MutationAgent(abctools.AbjadObject):
    r'''A wrapper around the Abjad mutation methods.

    ..  container:: example

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> show(staff) # doctest: +SKIP

        ::

            >>> mutate(staff[2:])
            MutationAgent(client=SliceSelection(Note("d'4"), Note("f'4")))

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_client',
        )

    ### INITIALIZER ###

    def __init__(self, client=None):
        self._client = client

    ### PUBLIC PROPERTIES ###

    @property
    def client(self):
        r'''Returns client of mutation agent.

        Returns selection or component.
        '''
        return self._client

    ### PUBLIC METHODS ###

    def copy(self, n=1, include_enclosing_containers=False):
        r'''Copies component and fractures crossing spanners.

        Returns new component.
        '''
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        if isinstance(self._client, scoretools.Component):
            selection = selectiontools.ContiguousSelection(self._client)
        else:
            selection = self._client
        result = selection._copy(
            n=n,
            include_enclosing_containers=include_enclosing_containers,
            )
        if isinstance(self._client, scoretools.Component):
            if len(result) == 1:
                result = result[0]
        return result

    def extract(self, scale_contents=False):
        r'''Extracts mutation client from score.

        Leaves children of mutation client in score.

        ..  container:: example

            **Example 1.** Extract tuplet:

            ::

                >>> staff = Staff()
                >>> time_signature = TimeSignature((3, 4))
                >>> attach(time_signature, staff)
                >>> staff.append(Tuplet((3, 2), "c'4 e'4"))
                >>> staff.append(Tuplet((3, 2), "d'4 f'4"))
                >>> hairpin = spannertools.Hairpin([], 'p < f')
                >>> attach(hairpin, staff.select_leaves())
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \time 3/4
                    \tweak #'text #tuplet-number::calc-fraction-text
                    \times 3/2 {
                        c'4 \< \p
                        e'4
                    }
                    \tweak #'text #tuplet-number::calc-fraction-text
                    \times 3/2 {
                        d'4
                        f'4 \f
                    }
                }

            ::

                >>> empty_tuplet = mutate(staff[-1]).extract()
                >>> empty_tuplet = mutate(staff[0]).extract()
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \time 3/4
                    c'4 \< \p
                    e'4
                    d'4
                    f'4 \f
                }

        ..  container:: example

            **Example 2.** Scale tuplet contents and then extract tuplet:

            ::

                >>> staff = Staff()
                >>> time_signature = TimeSignature((3, 4))
                >>> attach(time_signature, staff)
                >>> staff.append(Tuplet((3, 2), "c'4 e'4"))
                >>> staff.append(Tuplet((3, 2), "d'4 f'4"))
                >>> hairpin = spannertools.Hairpin([], 'p < f')
                >>> attach(hairpin, staff.select_leaves())
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \time 3/4
                    \tweak #'text #tuplet-number::calc-fraction-text
                    \times 3/2 {
                        c'4 \< \p
                        e'4
                    }
                    \tweak #'text #tuplet-number::calc-fraction-text
                    \times 3/2 {
                        d'4
                        f'4 \f
                    }
                }

            ::

                >>> empty_tuplet = mutate(staff[-1]).extract(
                ...     scale_contents=True)
                >>> empty_tuplet = mutate(staff[0]).extract(
                ...     scale_contents=True)
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \time 3/4
                    c'4. \< \p
                    e'4.
                    d'4.
                    f'4. \f
                }

        Returns mutation client.
        '''
        return self._client._extract(scale_contents=scale_contents)

    def fuse(self):
        r'''Fuses mutation client.

        ..  container:: example

            **Example 1.** Fuse in-score leaves:

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> show(staff) # doctest: +SKIP

            ::

                >>> mutate(staff[1:]).fuse()
                [Note("d'4.")]
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    c'8
                    d'4.
                }

        ..  container:: example

            **Example 2.** Fuse parent-contiguous fixed-duration tuplets
            in selection:

            ::

                >>> tuplet_1 = scoretools.FixedDurationTuplet(
                ...     Duration(2, 8), [])
                >>> tuplet_1.extend("c'8 d'8 e'8")
                >>> beam = spannertools.Beam()
                >>> attach(beam, tuplet_1[:])
                >>> duration = Duration(2, 16)
                >>> tuplet_2 = scoretools.FixedDurationTuplet(duration, [])
                >>> tuplet_2.extend("c'16 d'16 e'16")
                >>> slur = spannertools.Slur()
                >>> attach(slur, tuplet_2[:])
                >>> staff = Staff([tuplet_1, tuplet_2])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \times 2/3 {
                        c'8 [
                        d'8
                        e'8 ]
                    }
                    \times 2/3 {
                        c'16 (
                        d'16
                        e'16 )
                    }
                }

            ::

                >>> tuplets = staff[:]
                >>> mutate(tuplets).fuse()
                FixedDurationTuplet(Duration(3, 8), "c'8 d'8 e'8 c'16 d'16 e'16")
                >>> show(staff) #doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \times 2/3 {
                        c'8 [
                        d'8
                        e'8 ]
                        c'16 (
                        d'16
                        e'16 )
                    }
                }

            Returns new tuplet.

            Fuses zero or more parent-contiguous `tuplets`.

            Allows in-score `tuplets`.

            Allows outside-of-score `tuplets`.

            All `tuplets` must carry the same multiplier.

            All `tuplets` must be of the same type.

        ..  container:: example

            **Example 3.** Fuse in-score measures:

            ::

                >>> staff = Staff()
                >>> staff.append(Measure((1, 4), "c'8 d'8"))
                >>> staff.append(Measure((2, 8), "e'8 f'8"))
                >>> slur = spannertools.Slur()
                >>> attach(slur, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    {
                        \time 1/4
                        c'8 (
                        d'8
                    }
                    {
                        \time 2/8
                        e'8
                        f'8 )
                    }
                }

            ::

                >>> measures = staff[:]
                >>> mutate(measures).fuse()
                Measure((2, 4), "c'8 d'8 e'8 f'8")
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    {
                        \time 2/4
                        c'8 (
                        d'8
                        e'8
                        f'8 )
                    }
                }

        Returns fused mutation client.
        '''
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        if isinstance(self._client, scoretools.Component):
            selection = selectiontools.ContiguousSelection(self._client)
            return selection._fuse()
        elif isinstance(self._client, selectiontools.Selection) and \
            self._client._all_are_contiguous_components_in_same_logical_voice(
            self._client):
            selection = selectiontools.ContiguousSelection(self._client)
            return selection._fuse()

    def replace(self, recipients):
        r'''Replaces mutation client (and contents of mutation client)
        with `recipients`.

        ..  container:: example

            **Example 1.** Replace in-score tuplet (and children of tuplet)
            with notes. Functions exactly the same as container setitem:

                >>> tuplet_1 = Tuplet((2, 3), "c'4 d'4 e'4")
                >>> tuplet_2 = Tuplet((2, 3), "d'4 e'4 f'4")
                >>> staff = Staff([tuplet_1, tuplet_2])
                >>> hairpin = spannertools.Hairpin([], 'p < f')
                >>> attach(hairpin, staff[:])
                >>> slur = spannertools.Slur()
                >>> attach(slur, staff.select_leaves())
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \times 2/3 {
                        c'4 \< \p (
                        d'4
                        e'4
                    }
                    \times 2/3 {
                        d'4
                        e'4
                        f'4 \f )
                    }
                }

            ::

                >>> notes = scoretools.make_notes(
                ...     "c' d' e' f' c' d' e' f'",
                ...     Duration(1, 16),
                ...     )
                >>> mutate([tuplet_1]).replace(notes)
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    c'16 \< \p (
                    d'16
                    e'16
                    f'16
                    c'16
                    d'16
                    e'16
                    f'16
                    \times 2/3 {
                        d'4
                        e'4
                        f'4 \f )
                    }
                }

            Preserves both hairpin and slur.

        Returns none.
        '''
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        Selection = selectiontools.Selection
        if isinstance(self._client, selectiontools.SliceSelection):
            donors = self._client
        else:
            donors = selectiontools.SliceSelection(self._client)
        assert donors._all_are_contiguous_components_in_same_parent(donors)
        if not isinstance(recipients, selectiontools.Selection):
            recipients = selectiontools.SliceSelection(recipients)
        assert recipients._all_are_contiguous_components_in_same_parent(
            recipients)
        if donors:
            parent, start, stop = \
                donors._get_parent_and_start_stop_indices()
            assert parent is not None, repr(donors)
            parent.__setitem__(slice(start, stop + 1), recipients)

    def respell_with_flats(self):
        r'''Respell named pitches in mutation client with flats:

        ::

            >>> staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8")
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                c'8
                cs'8
                d'8
                ef'8
                e'8
                f'8
            }

        ::

            >>> mutate(staff).respell_with_flats()
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                c'8
                df'8
                d'8
                ef'8
                e'8
                f'8
            }

        Returns none.
        '''
        from abjad.tools import scoretools
        for leaf in iterate(self._client).by_class(scoretools.Leaf):
            if isinstance(leaf, scoretools.Chord):
                for note_head in leaf.note_heads:
                    note_head.written_pitch = \
                        note_head.written_pitch.respell_with_flats()
            elif hasattr(leaf, 'written_pitch'):
                leaf.written_pitch = leaf.written_pitch.respell_with_flats()

    def respell_with_sharps(self):
        r'''Respell named pitches in mutation client with sharps:

        ::

            >>> staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8")
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                c'8
                cs'8
                d'8
                ef'8
                e'8
                f'8
            }

        ::

            >>> mutate(staff).respell_with_sharps()
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                c'8
                cs'8
                d'8
                ds'8
                e'8
                f'8
            }

        Returns none.
        '''
        from abjad.tools import scoretools
        for leaf in iterate(self._client).by_class(scoretools.Leaf):
            if isinstance(leaf, scoretools.Chord):
                for note_head in leaf.note_heads:
                    note_head.written_pitch = \
                        note_head.written_pitch.respell_with_sharps()
            elif hasattr(leaf, 'written_pitch'):
                leaf.written_pitch = leaf.written_pitch.respell_with_sharps()

    def rewrite_meter(
        self,
        meter,
        boundary_depth=None,
        initial_offset=None,
        maximum_dot_count=None,
        ):
        r'''Rewrite the contents of logical ties in an expression to match
        a meter.

        ..  container:: example

            **Example 1.** Rewrite the contents of a measure in a staff
            using the default meter for that measure's time signature:

            ::

                >>> parseable = "abj: | 2/4 c'2 ~ |"
                >>> parseable += "| 4/4 c'32 d'2.. ~ d'16 e'32 ~ |"
                >>> parseable += "| 2/4 e'2 |"
                >>> staff = Staff(parseable)

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    {
                        \time 2/4
                        c'2 ~
                    }
                    {
                        \time 4/4
                        c'32
                        d'2.. ~
                        d'16
                        e'32 ~
                    }
                    {
                        \time 2/4
                        e'2
                    }
                }

            ::

                >>> show(staff) # doctest: +SKIP

            ::

                >>> meter = metertools.Meter((4, 4))
                >>> print meter.pretty_rtm_format
                (4/4 (
                    1/4
                    1/4
                    1/4
                    1/4))

            ::

                >>> mutate(staff[1][:]).rewrite_meter(meter)

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    {
                        \time 2/4
                        c'2 ~
                    }
                    {
                        \time 4/4
                        c'32
                        d'8.. ~
                        d'2 ~
                        d'8..
                        e'32 ~
                    }
                    {
                        \time 2/4
                        e'2
                    }
                }

            ::

                >>> show(staff) # doctest: +SKIP

        ..  container:: example

            **Example 2.** Rewrite the contents of a measure in a staff
            using a custom meter:

            ::

                >>> staff = Staff(parseable)

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    {
                        \time 2/4
                        c'2 ~
                    }
                    {
                        \time 4/4
                        c'32
                        d'2.. ~
                        d'16
                        e'32 ~
                    }
                    {
                        \time 2/4
                        e'2
                    }
                }

            ::

                >>> show(staff) # doctest: +SKIP

            ::

                >>> rtm = '(4/4 ((2/4 (1/4 1/4)) (2/4 (1/4 1/4))))'
                >>> meter = metertools.Meter(rtm)
                >>> print meter.pretty_rtm_format # doctest: +SKIP
                (4/4 (
                    (2/4 (
                        1/4
                        1/4))
                    (2/4 (
                        1/4
                        1/4))))

            ::

                >>> mutate(staff[1][:]).rewrite_meter(meter)

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    {
                        \time 2/4
                        c'2 ~
                    }
                    {
                        \time 4/4
                        c'32
                        d'4... ~
                        d'4...
                        e'32 ~
                    }
                    {
                        \time 2/4
                        e'2
                    }
                }

            ::

                >>> show(staff) # doctest: +SKIP

        ..  container:: example

            **Example 3.** Limit the maximum number of dots per leaf using
            `maximum_dot_count`:

            ::

                >>> parseable = "abj: | 3/4 c'32 d'8 e'8 fs'4... |"
                >>> measure = parse(parseable)

            ..  doctest::

                >>> print format(measure)
                {
                    \time 3/4
                    c'32
                    d'8
                    e'8
                    fs'4...
                }

            ::

                >>> show(measure) # doctest: +SKIP

            Without constraining the `maximum_dot_count`:

            ::

                >>> mutate(measure[:]).rewrite_meter(measure)

            ..  doctest::

                >>> print format(measure)
                {
                    \time 3/4
                    c'32
                    d'16. ~
                    d'32
                    e'16. ~
                    e'32
                    fs'4...
                }

            ::

                >>> show(measure) # doctest: +SKIP

            Constraining the `maximum_dot_count` to `2`:

            ::

                >>> measure = parse(parseable)
                >>> mutate(measure[:]).rewrite_meter(
                ...     measure,
                ...     maximum_dot_count=2,
                ...     )

            ..  doctest::

                >>> print format(measure)
                {
                    \time 3/4
                    c'32
                    d'16. ~
                    d'32
                    e'16. ~
                    e'32
                    fs'8.. ~
                    fs'4
                }

            ::

                >>> show(measure) # doctest: +SKIP

            Constraining the `maximum_dot_count` to `1`:

            ::

                >>> measure = parse(parseable)
                >>> mutate(measure[:]).rewrite_meter(
                ...     measure,
                ...     maximum_dot_count=1,
                ...     )

            ..  doctest::

                >>> print format(measure)
                {
                    \time 3/4
                    c'32
                    d'16. ~
                    d'32
                    e'16. ~
                    e'32
                    fs'16. ~
                    fs'8 ~
                    fs'4
                }

            ::

                >>> show(measure) # doctest: +SKIP

            Constraining the `maximum_dot_count` to `0`:

            ::

                >>> measure = parse(parseable)
                >>> mutate(measure[:]).rewrite_meter(
                ...     measure,
                ...     maximum_dot_count=0,
                ...     )

            ..  doctest::

                >>> print format(measure)
                {
                    \time 3/4
                    c'32
                    d'32 ~
                    d'16 ~
                    d'32
                    e'32 ~
                    e'16 ~
                    e'32
                    fs'32 ~
                    fs'16 ~
                    fs'8 ~
                    fs'4
                }

            ::

                >>> show(measure) # doctest: +SKIP

        ..  container:: example

            **Example 4.** Split logical ties at different depths of the
            `Meter`, if those logical ties cross any offsets at that
            depth, but do not also both begin and end at any of those offsets.

            Consider the default meter for `9/8`:

            ::

                >>> meter = metertools.Meter((9, 8))
                >>> print meter.pretty_rtm_format
                (9/8 (
                    (3/8 (
                        1/8
                        1/8
                        1/8))
                    (3/8 (
                        1/8
                        1/8
                        1/8))
                    (3/8 (
                        1/8
                        1/8
                        1/8))))

            We can establish that meter without specifying
            a `boundary_depth`:

            ::

                >>> parseable = "abj: | 9/8 c'2 d'2 e'8 |"
                >>> measure = parse(parseable)

            ..  doctest::

                >>> print format(measure)
                {
                    \time 9/8
                    c'2
                    d'2
                    e'8
                }

            ::

                >>> show(measure) # doctest: +SKIP

            ::

                >>> mutate(measure[:]).rewrite_meter(measure)

            ..  doctest::

                >>> print format(measure)
                {
                    \time 9/8
                    c'2
                    d'4 ~
                    d'4
                    e'8
                }

            ::

                >>> show(measure) # doctest: +SKIP

            With a `boundary_depth` of `1`, logical ties which cross any offsets
            created by nodes with a depth of `1` in this Meter's rhythm
            tree - i.e.  `0/8`, `3/8`, `6/8` and `9/8` - which do not also
            begin and end at any of those offsets, will be split:

            ::

                >>> measure = parse(parseable)
                >>> mutate(measure[:]).rewrite_meter(
                ...     measure,
                ...     boundary_depth=1,
                ...     )

            ..  doctest::

                >>> print format(measure)
                {
                    \time 9/8
                    c'4. ~
                    c'8
                    d'4 ~
                    d'4
                    e'8
                }

            ::

                >>> show(measure) # doctest: +SKIP

            For this `9/8` meter, and this input notation, A `boundary_depth`
            of `2` causes no change, as all logical ties already align to
            multiples of `1/8`:

            ::

                >>> measure = parse(parseable)
                >>> mutate(measure[:]).rewrite_meter(
                ...     measure,
                ...     boundary_depth=2,
                ...     )

            ..  doctest::

                >>> print format(measure)
                {
                    \time 9/8
                    c'2
                    d'4 ~
                    d'4
                    e'8
                }

            ::

                >>> show(measure) # doctest: +SKIP

        ..  container:: example

            **Example 5.** Comparison of `3/4` and `6/8`, at `boundary_depths`
            of 0 and 1:

            ::

                >>> triple = "abj: | 3/4 2 4 || 3/4 4 2 || 3/4 4. 4. |"
                >>> triple += "| 3/4 2 ~ 8 8 || 3/4 8 8 ~ 2 |"
                >>> duples = "abj: | 6/8 2 4 || 6/8 4 2 || 6/8 4. 4. |"
                >>> duples += "| 6/8 2 ~ 8 8 || 6/8 8 8 ~ 2 |"
                >>> score = Score([Staff(triple), Staff(duples)])

            In order to see the different time signatures on each staff,
            we need to move some engravers from the Score context to the
            Staff context:

            ::

                >>> engravers = [
                ...     'Timing_translator',
                ...     'Time_signature_engraver',
                ...     'Default_bar_line_engraver',
                ...     ]
                >>> score.engraver_removals.extend(engravers)
                >>> score[0].engraver_consists.extend(engravers)
                >>> score[1].engraver_consists.extend(engravers)

            ..  doctest::

                >>> print format(score)
                \new Score \with {
                    \remove Timing_translator
                    \remove Time_signature_engraver
                    \remove Default_bar_line_engraver
                } <<
                    \new Staff \with {
                        \consists Timing_translator
                        \consists Time_signature_engraver
                        \consists Default_bar_line_engraver
                    } {
                        {
                            \time 3/4
                            c'2
                            c'4
                        }
                        {
                            c'4
                            c'2
                        }
                        {
                            c'4.
                            c'4.
                        }
                        {
                            c'2 ~
                            c'8
                            c'8
                        }
                        {
                            c'8
                            c'8 ~
                            c'2
                        }
                    }
                    \new Staff \with {
                        \consists Timing_translator
                        \consists Time_signature_engraver
                        \consists Default_bar_line_engraver
                    } {
                        {
                            \time 6/8
                            c'2
                            c'4
                        }
                        {
                            c'4
                            c'2
                        }
                        {
                            c'4.
                            c'4.
                        }
                        {
                            c'2 ~
                            c'8
                            c'8
                        }
                        {
                            c'8
                            c'8 ~
                            c'2
                        }
                    }
                >>

            ::

                >>> show(score) # doctest: +SKIP

            Here we establish a meter without specifying and boundary
            depth:

            ::

                >>> for measure in iterate(score).by_class(scoretools.Measure):
                ...     mutate(measure[:]).rewrite_meter(measure)

            ..  doctest::

                >>> print format(score)
                \new Score \with {
                    \remove Timing_translator
                    \remove Time_signature_engraver
                    \remove Default_bar_line_engraver
                } <<
                    \new Staff \with {
                        \consists Timing_translator
                        \consists Time_signature_engraver
                        \consists Default_bar_line_engraver
                    } {
                        {
                            \time 3/4
                            c'2
                            c'4
                        }
                        {
                            c'4
                            c'2
                        }
                        {
                            c'4.
                            c'4.
                        }
                        {
                            c'2 ~
                            c'8
                            c'8
                        }
                        {
                            c'8
                            c'8 ~
                            c'2
                        }
                    }
                    \new Staff \with {
                        \consists Timing_translator
                        \consists Time_signature_engraver
                        \consists Default_bar_line_engraver
                    } {
                        {
                            \time 6/8
                            c'2
                            c'4
                        }
                        {
                            c'4
                            c'2
                        }
                        {
                            c'4.
                            c'4.
                        }
                        {
                            c'4. ~
                            c'4
                            c'8
                        }
                        {
                            c'8
                            c'4 ~
                            c'4.
                        }
                    }
                >>

            ::

                >>> show(score) # doctest: +SKIP

            Here we re-establish meter at a boundary depth of `1`:

            ::

                >>> for measure in iterate(score).by_class(scoretools.Measure):
                ...     mutate(measure[:]).rewrite_meter(
                ...         measure,
                ...         boundary_depth=1,
                ...         )
                ...

            ..  doctest::

                >>> print format(score)
                \new Score \with {
                    \remove Timing_translator
                    \remove Time_signature_engraver
                    \remove Default_bar_line_engraver
                } <<
                    \new Staff \with {
                        \consists Timing_translator
                        \consists Time_signature_engraver
                        \consists Default_bar_line_engraver
                    } {
                        {
                            \time 3/4
                            c'2
                            c'4
                        }
                        {
                            c'4
                            c'2
                        }
                        {
                            c'4 ~
                            c'8
                            c'8 ~
                            c'4
                        }
                        {
                            c'2 ~
                            c'8
                            c'8
                        }
                        {
                            c'8
                            c'8 ~
                            c'2
                        }
                    }
                    \new Staff \with {
                        \consists Timing_translator
                        \consists Time_signature_engraver
                        \consists Default_bar_line_engraver
                    } {
                        {
                            \time 6/8
                            c'4. ~
                            c'8
                            c'4
                        }
                        {
                            c'4
                            c'8 ~
                            c'4.
                        }
                        {
                            c'4.
                            c'4.
                        }
                        {
                            c'4. ~
                            c'4
                            c'8
                        }
                        {
                            c'8
                            c'4 ~
                            c'4.
                        }
                    }
                >>

            ::

                >>> show(score) # doctest: +SKIP

            Note that the two time signatures are much more clearly
            disambiguated above.

        ..  container:: example

            **Example 6.** Establishing meter recursively in measures
            with nested tuplets:

            ::

                >>> parseable = "abj: | 4/4 c'16 ~ c'4 d'8. ~ "
                >>> parseable += "2/3 { d'8. ~ 3/5 { d'16 e'8. f'16 ~ } } "
                >>> parseable += "f'4 |"
                >>> measure = parse(parseable)

            ..  doctest::

                >>> print format(measure)
                {
                    \time 4/4
                    c'16 ~
                    c'4
                    d'8. ~
                    \times 2/3 {
                        d'8. ~
                        \tweak #'text #tuplet-number::calc-fraction-text
                        \times 3/5 {
                            d'16
                            e'8.
                            f'16 ~
                        }
                    }
                    f'4
                }

            ::

                >>> show(measure) # doctest: +SKIP

            When establishing a meter on a selection of components
            which contain containers, like `Tuplets` or `Containers`,
            `metertools.rewrite_meter()` will recurse into
            those containers, treating them as measures whose time
            signature is derived from the preprolated preprolated_duration
            of the container's contents:

            ::

                >>> mutate(measure[:]).rewrite_meter(
                ...     measure,
                ...     boundary_depth=1,
                ...     )

            ..  doctest::

                >>> print format(measure)
                {
                    \time 4/4
                    c'4 ~
                    c'16
                    d'8. ~
                    \times 2/3 {
                        d'8 ~
                        d'16 ~
                        \tweak #'text #tuplet-number::calc-fraction-text
                        \times 3/5 {
                            d'16
                            e'8 ~
                            e'16
                            f'16 ~
                        }
                    }
                    f'4
                }

            ::

                >>> show(measure) # doctest: +SKIP

        Operates in place and returns none.
        '''
        from abjad.tools import selectiontools
        from abjad.tools.metertools._rewrite_meter \
            import _rewrite_meter
        assert isinstance(self._client, selectiontools.Selection)
        result = _rewrite_meter(
            self._client,
            meter,
            boundary_depth=boundary_depth,
            initial_offset=initial_offset,
            maximum_dot_count=maximum_dot_count,
            )
        return result

    def scale(self, multiplier):
        r'''Scales mutation client by `multiplier`.

        ..  container:: example

            **Example 1a.** Scale note duration by dot-generating multiplier:

            ::

                >>> staff = Staff("c'8 ( d'8 e'8 f'8 )")
                >>> show(staff) # doctest: +SKIP

            ::

                >>> mutate(staff[1]).scale(Multiplier(3, 2))
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    c'8 (
                    d'8.
                    e'8
                    f'8 )
                }

        ..  container:: example

            **Example 1b.** Scale nontrivial logical tie
            by dot-generating `multiplier`:

            ::

                >>> staff = Staff(r"c'8 \accent ~ c'8 d'8")
                >>> time_signature = TimeSignature((3, 8))
                >>> attach(time_signature, staff)
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \time 3/8
                    c'8 -\accent ~
                    c'8
                    d'8
                }

            ::

                >>> logical_tie = inspect(staff[0]).get_logical_tie()
                >>> logical_tie = mutate(logical_tie).scale(Multiplier(3, 2))
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \time 3/8
                    c'4. -\accent
                    d'8
                }

        ..  container:: example

            **Example 1c.** Scale container by dot-generating multiplier:

            ::

                >>> container = Container(r"c'8 ( d'8 e'8 f'8 )")
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    c'8 (
                    d'8
                    e'8
                    f'8 )
                }

            ::

                >>> mutate(container).scale(Multiplier(3, 2))
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    c'8. (
                    d'8.
                    e'8.
                    f'8. )
                }

        ..  container:: example

            **Example 2a.** Scale note by tie-generating multiplier:

            ::

                >>> staff = Staff("c'8 ( d'8 e'8 f'8 )")
                >>> show(staff) # doctest: +SKIP

            ::

                >>> mutate(staff[1]).scale(Multiplier(5, 4))
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    c'8 (
                    d'8 ~
                    d'32
                    e'8
                    f'8 )
                }

        ..  container:: example

            **Example 2b.** Scale nontrivial logical tie
            by tie-generating `multiplier`:

            ::

                >>> staff = Staff(r"c'8 \accent ~ c'8 d'16")
                >>> time_signature = TimeSignature((5, 16))
                >>> attach(time_signature, staff)
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \time 5/16
                    c'8 -\accent ~
                    c'8
                    d'16
                }

            ::

                >>> logical_tie = inspect(staff[0]).get_logical_tie()
                >>> logical_tie = mutate(logical_tie).scale(Multiplier(5, 4))
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \time 5/16
                    c'4 -\accent ~
                    c'16
                    d'16
                }

        ..  container:: example

            **Example 2c.** Scale container by tie-generating multiplier:

            ::

                >>> container = Container(r"c'8 ( d'8 e'8 f'8 )")
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    c'8 (
                    d'8
                    e'8
                    f'8 )
                }

            ::

                >>> mutate(container).scale(Multiplier(5, 4))
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    c'8 ( ~
                    c'32
                    d'8 ~
                    d'32
                    e'8 ~
                    e'32
                    f'8 ~
                    f'32 )
                }

        ..  container:: example

            **Example 3a.** Scale note by tuplet-generating multiplier:

            ::

                >>> staff = Staff("c'8 ( d'8 e'8 f'8 )")
                >>> show(staff) # doctest: +SKIP

            ::

                >>> mutate(staff[1]).scale(Multiplier(2, 3))
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    c'8 (
                    \times 2/3 {
                        d'8
                    }
                    e'8
                    f'8 )
                }

        ..  container:: example

            **Example 3b.** Scale trivial logical tie
            by tuplet-generating multiplier:

            ::

                >>> staff = Staff(r"c'8 \accent")
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    c'8 -\accent
                }

            ::

                >>> logical_tie = inspect(staff[0]).get_logical_tie()
                >>> logical_tie = mutate(logical_tie).scale(Multiplier(4, 3))
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \times 2/3 {
                        c'4 -\accent
                    }
                }

        ..  container:: example

            **Example 3c.** Scale container by tuplet-generating multiplier:

            ::

                >>> container = Container(r"c'8 ( d'8 e'8 f'8 )")
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    c'8 (
                    d'8
                    e'8
                    f'8 )
                }

            ::

                >>> mutate(container).scale(Multiplier(4, 3))
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print format(container)
                {
                    \times 2/3 {
                        c'4 (
                    }
                    \times 2/3 {
                        d'4
                    }
                    \times 2/3 {
                        e'4
                    }
                    \times 2/3 {
                        f'4 )
                    }
                }

        ..  container:: example

            **Example 4.** Scale note by tie- and tuplet-generating
            multiplier:

            ::

                >>> staff = Staff("c'8 ( d'8 e'8 f'8 )")
                >>> show(staff) # doctest: +SKIP

            ::

                >>> mutate(staff[1]).scale(Multiplier(5, 6))
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    c'8 (
                    \times 2/3 {
                        d'8 ~
                        d'32
                    }
                    e'8
                    f'8 )
                }

        ..  container:: example

            **Example 5.** Scale note carrying LilyPond multiplier:

            ::

                >>> note = Note("c'8")
                >>> attach(Multiplier(1, 2), note)
                >>> show(note) # doctest: +SKIP

            ..  doctest::

                >>> print format(note)
                c'8 * 1/2

            ::

                >>> mutate(note).scale(Multiplier(5, 3))
                >>> show(note) # doctest: +SKIP

            ..  doctest::

                >>> print format(note)
                c'8 * 5/6

        ..  container:: example

            **Example 6.** Scale tuplet:

            ::

                >>> staff = Staff()
                >>> time_signature = TimeSignature((4, 8))
                >>> attach(time_signature, staff)
                >>> tuplet = scoretools.Tuplet((4, 5), [])
                >>> tuplet.extend("c'8 d'8 e'8 f'8 g'8")
                >>> staff.append(tuplet)
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \time 4/8
                    \times 4/5 {
                        c'8
                        d'8
                        e'8
                        f'8
                        g'8
                    }
                }

            ::

                >>> mutate(tuplet).scale(Multiplier(2))
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \time 4/8
                    \times 4/5 {
                        c'4
                        d'4
                        e'4
                        f'4
                        g'4
                    }
                }

        ..  container:: example

            **Example 7.** Scale fixed-duration tuplet:

            ::

                >>> staff = Staff()
                >>> time_signature = TimeSignature((4, 8))
                >>> attach(time_signature, staff)
                >>> tuplet = scoretools.FixedDurationTuplet((4, 8), [])
                >>> tuplet.extend("c'8 d'8 e'8 f'8 g'8")
                >>> staff.append(tuplet)
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \time 4/8
                    \times 4/5 {
                        c'8
                        d'8
                        e'8
                        f'8
                        g'8
                    }
                }

            ::

                >>> mutate(tuplet).scale(Multiplier(2))
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \time 4/8
                    \times 4/5 {
                        c'4
                        d'4
                        e'4
                        f'4
                        g'4
                    }
                }

        Returns none.
        '''
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        if hasattr(self._client, '_scale'):
            self._client._scale(multiplier)
        else:
            assert isinstance(self._client, selectiontools.Selection)
            for component in self._client:
                component._scale(multiplier)

    def splice(
        self,
        components,
        direction=Right,
        grow_spanners=True,
        ):
        r'''Splices `components` to the right or left of selection.

        Returns list of components.
        '''
        return self._client._splice(
            components,
            direction=direction,
            grow_spanners=grow_spanners,
            )

    # TODO: fix bug that unintentionally fractures ties.
    # TODO: add tests of tupletted notes and rests.
    # TODO: add examples that show indicator handling.
    # TODO: add example showing grace and after grace handling.
    def split(
        self,
        durations,
        fracture_spanners=False,
        cyclic=False,
        tie_split_notes=True,
        ):
        r'''Splits component or selection by `durations`.

        ..  container:: example

            **Example 1.** Split leaves:

            ::

                >>> staff = Staff("c'8 e' d' f' c' e' d' f'")
                >>> leaves = staff.select_leaves()
                >>> hairpin = spannertools.Hairpin(descriptor='p < f')
                >>> attach(hairpin, leaves)
                >>> override(staff).dynamic_line_spanner.staff_padding = 3
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff \with {
                    \override DynamicLineSpanner #'staff-padding = #3
                } {
                    c'8 \< \p
                    e'8
                    d'8
                    f'8
                    c'8
                    e'8
                    d'8
                    f'8 \f
                }

            ::

                >>> durations = [Duration(3, 16), Duration(7, 32)]
                >>> result = mutate(leaves).split(
                ...     durations,
                ...     tie_split_notes=False,
                ...     )
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff \with {
                    \override DynamicLineSpanner #'staff-padding = #3
                } {
                    c'8 \< \p
                    e'16
                    e'16
                    d'8
                    f'32
                    f'16.
                    c'8
                    e'8
                    d'8
                    f'8 \f
                }

        ..  container:: example

            **Example 2.** Split leaves and fracture crossing spanners:

            ::

                >>> staff = Staff("c'8 e' d' f' c' e' d' f'")
                >>> leaves = staff.select_leaves()
                >>> hairpin = spannertools.Hairpin(descriptor='p < f')
                >>> attach(hairpin, leaves)
                >>> override(staff).dynamic_line_spanner.staff_padding = 3
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff \with {
                    \override DynamicLineSpanner #'staff-padding = #3
                } {
                    c'8 \< \p
                    e'8
                    d'8
                    f'8
                    c'8
                    e'8
                    d'8
                    f'8 \f
                }

            ::

                >>> durations = [Duration(3, 16), Duration(7, 32)]
                >>> result = mutate(leaves).split(
                ...     durations,
                ...     fracture_spanners=True,
                ...     tie_split_notes=False,
                ...     )
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff \with {
                    \override DynamicLineSpanner #'staff-padding = #3
                } {
                    c'8 \< \p
                    e'16 \f
                    e'16 \< \p
                    d'8
                    f'32 \f
                    f'16. \< \p
                    c'8
                    e'8
                    d'8
                    f'8 \f
                }

        ..  container:: example

            **Example 3.** Split leaves cyclically:

            ::

                >>> staff = Staff("c'8 e' d' f' c' e' d' f'")
                >>> leaves = staff.select_leaves()
                >>> hairpin = spannertools.Hairpin(descriptor='p < f')
                >>> attach(hairpin, leaves)
                >>> override(staff).dynamic_line_spanner.staff_padding = 3
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff \with {
                    \override DynamicLineSpanner #'staff-padding = #3
                } {
                    c'8 \< \p
                    e'8
                    d'8
                    f'8
                    c'8
                    e'8
                    d'8
                    f'8 \f
                }

            ::

                >>> durations = [Duration(3, 16), Duration(7, 32)]
                >>> result = mutate(leaves).split(
                ...     durations,
                ...     cyclic=True,
                ...     tie_split_notes=False,
                ...     )
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff \with {
                    \override DynamicLineSpanner #'staff-padding = #3
                } {
                    c'8 \< \p
                    e'16
                    e'16
                    d'8
                    f'32
                    f'16.
                    c'16.
                    c'32
                    e'8
                    d'16
                    d'16
                    f'8 \f
                }

        ..  container:: example

            **Example 4.** Split leaves cyclically and fracture spanners:

            ::

                >>> staff = Staff("c'8 e' d' f' c' e' d' f'")
                >>> leaves = staff.select_leaves()
                >>> hairpin = spannertools.Hairpin(descriptor='p < f')
                >>> attach(hairpin, leaves)
                >>> override(staff).dynamic_line_spanner.staff_padding = 3
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff \with {
                    \override DynamicLineSpanner #'staff-padding = #3
                } {
                    c'8 \< \p
                    e'8
                    d'8
                    f'8
                    c'8
                    e'8
                    d'8
                    f'8 \f
                }

            ::

                >>> durations = [Duration(3, 16), Duration(7, 32)]
                >>> result = mutate(leaves).split(
                ...     durations,
                ...     cyclic=True,
                ...     fracture_spanners=True,
                ...     tie_split_notes=False,
                ...     )
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff \with {
                    \override DynamicLineSpanner #'staff-padding = #3
                } {
                    c'8 \< \p
                    e'16 \f
                    e'16 \< \p
                    d'8
                    f'32 \f
                    f'16. \< \p
                    c'16. \f
                    c'32 \< \p
                    e'8
                    d'16 \f
                    d'16 \< \p
                    f'8 \f
                }

        ..  container:: example

            **Example 5.** Split tupletted leaves and fracture
                crossing spanners:

            ::

                >>> staff = Staff()
                >>> staff.append(Tuplet((2, 3), "c'4 d' e'"))
                >>> staff.append(Tuplet((2, 3), "c'4 d' e'"))
                >>> leaves = staff.select_leaves()
                >>> slur = spannertools.Slur()
                >>> attach(slur, leaves)
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \times 2/3 {
                        c'4 (
                        d'4
                        e'4
                    }
                    \times 2/3 {
                        c'4
                        d'4
                        e'4 )
                    }
                }

            ::

                >>> durations = [Duration(1, 4)]
                >>> result = mutate(leaves).split(
                ...     durations,
                ...     fracture_spanners=True,
                ...     tie_split_notes=False,
                ...     )
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \times 2/3 {
                        c'4 (
                        d'8 )
                        d'8 (
                        e'4
                    }
                    \times 2/3 {
                        c'4
                        d'4
                        e'4 )
                    }
                }

        Returns list of selections.
        '''
        from abjad.tools import scoretools
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        # check input
        components = self._client
        single_component_input = False
        if isinstance(components, scoretools.Component):
            single_component_input = True
            components = selectiontools.Selection(components)
        assert all(
            isinstance(x, scoretools.Component) for x in components)
        if not isinstance(components, selectiontools.Selection):
            components = selectiontools.Selection(components)
        durations = [durationtools.Duration(x) for x in durations]
        # return if no split to be done
        if not durations:
            if single_component_input:
                return components
            else:
                return [], components
        # calculate total component duration
        total_component_duration = components.get_duration()
        total_split_duration = sum(durations)
        # calculate durations
        if cyclic:
            durations = sequencetools.repeat_sequence_to_weight_exactly(
                durations, total_component_duration)
        elif total_split_duration < total_component_duration:
            final_offset = total_component_duration - sum(durations)
            durations.append(final_offset)
        elif total_component_duration < total_split_duration:
            durations = sequencetools.truncate_sequence_to_weight(
                durations, total_component_duration)
        # keep copy of durations to partition result components
        durations_copy = durations[:]
        # calculate total split duration
        total_split_duration = sum(durations)
        assert total_split_duration == total_component_duration
        # initialize loop variables
        result, shard = [], []
        offset_index, offset_count = 0, len(durations)
        current_shard_duration = durationtools.Duration(0)
        remaining_components = list(components[:])
        advance_to_next_offset = True
        # loop and build shards by grabbing next component
        # and next duration each time through loop
        while True:
            # grab next split point
            if advance_to_next_offset:
                if durations:
                    next_split_point = durations.pop(0)
                else:
                    break
            advance_to_next_offset = True
            # grab next component from input stack of components
            if remaining_components:
                current_component = remaining_components.pop(0)
            else:
                break
            # find where current component endpoint will position us
            candidate_shard_duration = current_shard_duration + \
                current_component._get_duration()
            # if current component would fill current shard exactly
            if candidate_shard_duration == next_split_point:
                shard.append(current_component)
                result.append(shard)
                shard = []
                current_shard_duration = durationtools.Duration(0)
                offset_index += 1
            # if current component would exceed current shard
            elif next_split_point < candidate_shard_duration:
                local_split_duration = \
                    next_split_point - current_shard_duration
                if isinstance(current_component, scoretools.Leaf):
                    leaf_split_durations = [local_split_duration]
                    current_duration = current_component._get_duration()
                    additional_required_duration = \
                        current_duration - local_split_duration
                    split_durations = sequencetools.split_sequence_by_weights(
                        durations,
                        [additional_required_duration],
                        cyclic=False,
                        overhang=True,
                        )
                    additional_durations = split_durations[0]
                    leaf_split_durations.extend(additional_durations)
                    durations = split_durations[-1]
                    leaf_shards = current_component._split(
                        leaf_split_durations,
                        cyclic=False,
                        fracture_spanners=fracture_spanners,
                        tie_split_notes=tie_split_notes,
                        )
                    shard.extend(leaf_shards)
                    result.append(shard)
                    offset_index += len(additional_durations)
                else:
                    left_list, right_list = \
                        current_component._split_by_duration(
                        local_split_duration,
                        fracture_spanners=fracture_spanners,
                        tie_split_notes=tie_split_notes,
                        )
                    shard.extend(left_list)
                    result.append(shard)
                    remaining_components.__setitem__(slice(0, 0), right_list)
                shard = []
                offset_index += 1
                current_shard_duration = durationtools.Duration(0)
            # if current component would not fill current shard
            elif candidate_shard_duration < next_split_point:
                shard.append(current_component)
                current_shard_duration += current_component._get_duration()
                advance_to_next_offset = False
            else:
                raise ValueError
        # append any stub shard
        if len(shard):
            result.append(shard)
        # append any unexamined components
        if len(remaining_components):
            result.append(remaining_components)
        # partition split components according to input durations
        result = sequencetools.flatten_sequence(result)
        result = selectiontools.ContiguousSelection(result)
        result = result.partition_by_durations_exactly(durations_copy)
        # return list of shards
        result = [selectiontools.Selection(x) for x in result]
        return result

    def swap(self, container):
        r'''Swaps mutation client for empty `container`.

        ..  container:: example

            **Example 1.** Swap measures for tuplet:

                >>> staff = Staff()
                >>> staff.append(Measure((3, 4), "c'4 d'4 e'4"))
                >>> staff.append(Measure((3, 4), "d'4 e'4 f'4"))
                >>> leaves = staff.select_leaves()
                >>> hairpin = spannertools.Hairpin([], 'p < f')
                >>> attach(hairpin, leaves)
                >>> measures = staff[:]
                >>> slur = spannertools.Slur()
                >>> attach(slur, measures)
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    {
                        \time 3/4
                        c'4 \< \p (
                        d'4
                        e'4
                    }
                    {
                        d'4
                        e'4
                        f'4 \f )
                    }
                }

            ::

                >>> measures = staff[:]
                >>> tuplet = Tuplet(Multiplier(2, 3), [])
                >>> tuplet.preferred_denominator = 4
                >>> mutate(measures).swap(tuplet)
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \times 4/6 {
                        c'4 \< \p (
                        d'4
                        e'4
                        d'4
                        e'4
                        f'4 \f )
                    }
                }

        Returns none.
        '''
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        Selection = selectiontools.Selection
        if isinstance(self._client, selectiontools.SliceSelection):
            donors = self._client
        else:
            donors = selectiontools.SliceSelection(self._client)
        assert donors._all_are_contiguous_components_in_same_parent(donors)
        assert isinstance(container, scoretools.Container)
        assert not container, repr(container)
        donors._give_music_to_empty_container(container)
        donors._give_dominant_spanners([container])
        donors._give_position_in_parent_to_container(container)
